@@ -33,22 +33,20 @@
     into your KDE Addressbook.
 */
 
-#include <QFile>
-//Added by qt3to4:
-#include <QTextStream>
+#include "ldif_xxport.h"
 
+#include <QtCore/QFile>
+#include <QtCore/QTextStream>
+
+#include <kabc/ldifconverter.h>
+#include <kcodecs.h>
+#include <kdebug.h>
 #include <kfiledialog.h>
 #include <kio/netaccess.h>
 #include <klocale.h>
-#include <kcodecs.h>
 #include <kmessagebox.h>
 #include <ktemporaryfile.h>
 #include <kurl.h>
-#include <kabc/ldifconverter.h>
-
-#include <kdebug.h>
-
-#include "ldif_xxport.h"
 
 K_EXPORT_KADDRESSBOOK_XXFILTER( kaddrbk_ldif_xxport, LDIFXXPort )
 
@@ -71,7 +69,7 @@ KABC::Addressee::List LDIFXXPort::importContacts( const QString& ) const
     return addrList;
 
   QFile file( fileName );
-  if ( !file.open( QIODevice::ReadOnly ) ) {
+  if ( !file.open( QIODevice::ReadOnly | QIODevice::Text ) ) {
     QString msg = i18n( "<qt>Unable to open <b>%1</b> for reading.</qt>", fileName );
     KMessageBox::error( parentWidget(), msg );
     return addrList;
