@@ -7,41 +7,46 @@
     modify it under the terms of the GNU Library General Public
     License as published by the Free Software Foundation; either
     version 2 of the License, or (at your option) any later version.
-    
+
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Library General Public License for more details.
-    
+
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
     the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
     Boston, MA 02110-1301, USA.
 */
 
+
 #include "distributionlistdialog.h"
 
-#include <libemailfunctions/email.h>
+#include <kpimutils/email.h>
 #include <kabc/resource.h>
 #include <kabc/stdaddressbook.h>
 #include <kabc/distributionlist.h>
 
-#include <klistview.h>
-#include <klocale.h>
-#include <kdebug.h>
-#include <kmessagebox.h>
-#include <kinputdialog.h>
+#include <KLocale>
+#include <KDebug>
+#include <KLineEdit>
+#include <KMessageBox>
+#include <KInputDialog>
 
-#include <qlayout.h>
-#include <qlabel.h>
-#include <qlineedit.h>
+#include <QLabel>
+#include <QTreeWidget>
+#include <QTreeWidgetItem>
+#include <QVBoxLayout>
+#include <QHBoxLayout>
+#include "distributionlistdialog.moc"
 
-class DistributionListItem : public QCheckListItem
+class DistributionListItem : public QTreeWidgetItem
 {
   public:
-    DistributionListItem( QListView *list )
-      : QCheckListItem( list, QString::null, CheckBox )
+    DistributionListItem( QTreeWidget *tree )
+      : QTreeWidgetItem( tree )
     {
+      setFlags( flags() | Qt::ItemIsUserCheckable );
     }
 
     void setAddressee( const KABC::Addressee &a, const QString &email )
@@ -60,25 +65,25 @@ class DistributionListItem : public QCheckListItem
     {
       mAddressee = a;
       mEmail = email;
-      setText( 1, mAddressee.realName() );
-      setText( 2, mEmail );
+      setText( 0, mAddressee.realName() );
+      setText( 1, mEmail );
     }
 
     KABC::Addressee addressee() const
     {
       return mAddressee;
     }
-    
+
     QString email() const
     {
       return mEmail;
     }
-    
+
     bool isTransient() const
     {
       return mIsTransient;
     }
-    
+
   private:
     KABC::Addressee mAddressee;
     QString mEmail;
@@ -87,37 +92,51 @@ class DistributionListItem : public QCheckListItem
 
 
 DistributionListDialog::DistributionListDialog( QWidget *parent )
-  : KDialogBase( Plain, i18n("Save Distribution List"), User1 | Cancel,
-                 User1, parent, 0, false, false, i18n("Save List") )
+  : KDialog( parent )
 {
-  QFrame *topFrame = plainPage();
-  
+  QFrame *topFrame = new QFrame( this );
+  setMainWidget( topFrame );
+  setCaption( i18nc("@title:window", "Save Distribution List") );
+  setButtons( User1 | Cancel );
+  setDefaultButton( User1 );
+  setModal( false );
+  setButtonText( User1, i18nc("@action:button","Save List") );
+
   QBoxLayout *topLayout = new QVBoxLayout( topFrame );
   topLayout->setSpacing( spacingHint() );
-  
-  QBoxLayout *titleLayout = new QHBoxLayout( topLayout );
-  
-  QLabel *label = new QLabel( i18n("Name:"), topFrame );
+
+  QBoxLayout *titleLayout = new QHBoxLayout();
+  titleLayout->setSpacing( spacingHint() );
+  topLayout->addItem( titleLayout );
+
+  QLabel *label = new QLabel(
+    i18nc("@label:textbox Name of the distribution list.", "&Name:"), topFrame );
   titleLayout->addWidget( label );
-  
-  mTitleEdit = new QLineEdit( topFrame );
+
+  mTitleEdit = new KLineEdit( topFrame );
   titleLayout->addWidget( mTitleEdit );
   mTitleEdit->setFocus();
-  
-  mRecipientsList = new KListView( topFrame );
-  mRecipientsList->addColumn( QString::null );
-  mRecipientsList->addColumn( i18n("Name") );
-  mRecipientsList->addColumn( i18n("Email") );
+  mTitleEdit->setClearButtonShown( true );
+  label->setBuddy( mTitleEdit );
+
+  mRecipientsList = new QTreeWidget( topFrame );
+  mRecipientsList->setHeaderLabels(
+                     QStringList() << i18nc( "@title:column Name of the recipient","Name" )
+                                   << i18nc( "@title:column Email of the recipient", "Email" )
+                                  );
+  mRecipientsList->setRootIsDecorated( false );
   topLayout->addWidget( mRecipientsList );
+  connect( this, SIGNAL( user1Clicked() ),
+           this, SLOT( slotUser1() ) );
 }
 
 void DistributionListDialog::setRecipients( const Recipient::List &recipients )
 {
   Recipient::List::ConstIterator it;
-  for( it = recipients.begin(); it != recipients.end(); ++it ) {
-    QStringList emails = KPIM::splitEmailAddrList( (*it).email() );
+  for( it = recipients.constBegin(); it != recipients.constEnd(); ++it ) {
+    QStringList emails = KPIMUtils::splitAddressList( (*it).email() );
     QStringList::ConstIterator it2;
-    for( it2 = emails.begin(); it2 != emails.end(); ++it2 ) {
+    for( it2 = emails.constBegin(); it2 != emails.constEnd(); ++it2 ) {
       QString name;
       QString email;
       KABC::Addressee::parseEmailAddress( *it2, name, email );
@@ -130,12 +149,12 @@ void DistributionListDialog::setRecipients( const Recipient::List &recipients )
           a.setNameFromString( name );
           a.insertEmail( email );
           item->setTransientAddressee( a, email );
-          item->setOn( true );
+          item->setCheckState( 0, Qt::Checked );
         } else {
           KABC::Addressee::List::ConstIterator it3;
-          for( it3 = addressees.begin(); it3 != addressees.end(); ++it3 ) {
+          for( it3 = addressees.constBegin(); it3 != addressees.constEnd(); ++it3 ) {
             item->setAddressee( *it3, email );
-            if ( it3 == addressees.begin() ) item->setOn( true );
+            if ( it3 == addressees.constBegin() ) item->setCheckState( 0, Qt::Checked );
           }
         }
       }
@@ -149,50 +168,47 @@ void DistributionListDialog::slotUser1()
 
   KABC::AddressBook *ab = KABC::StdAddressBook::self( true );
 
-  QListViewItem *i = mRecipientsList->firstChild();
-  while( i ) {
-    DistributionListItem *item = static_cast<DistributionListItem *>( i );
-    if ( item->isOn() ) {
+  for (int i = 0; i < mRecipientsList->topLevelItemCount(); ++i) {
+    DistributionListItem *item = static_cast<DistributionListItem *>(
+        mRecipientsList->topLevelItem( i ));
+    if ( item && item->checkState( 0 ) == Qt::Checked ) {
       isEmpty = false;
       break;
     }
-    i = i->nextSibling();
   }
 
   if ( isEmpty ) {
     KMessageBox::information( this,
-                              i18n("There are no recipients in your list. "
+                              i18nc("@info", "There are no recipients in your list. "
                                    "First select some recipients, "
                                    "then try again.") );
     return;
   }
 
-  KABC::DistributionListManager manager( ab );
-  manager.load();
-
   QString name = mTitleEdit->text();
 
   if ( name.isEmpty() ) {
     bool ok = false;
-    name = KInputDialog::getText( i18n("New Distribution List"),
-      i18n("Please enter name:"), QString::null, &ok, this );
+    name = KInputDialog::getText( i18nc("@title:window","New Distribution List"),
+      i18nc("@label:textbox","Please enter name:"), QString(), &ok, this );
     if ( !ok || name.isEmpty() )
       return;
   }
 
-  if ( manager.list( name ) ) {
+  if ( ab->findDistributionListByName( name ) ) {
     KMessageBox::information( this,
-      i18n( "<qt>Distribution list with the given name <b>%1</b> "
-        "already exists. Please select a different name.</qt>" ).arg( name ) );
+      i18nc( "@info", "<para>Distribution list with the given name <resource>%1</resource> "
+        "already exists. Please select a different name.</para>", name ) );
     return;
   }
 
-  KABC::DistributionList *dlist = new KABC::DistributionList( &manager, name );
-  i = mRecipientsList->firstChild();
-  while( i ) {
-    DistributionListItem *item = static_cast<DistributionListItem *>( i );
-    if ( item->isOn() ) {
-      kdDebug() << "  " << item->addressee().fullEmail() << endl;
+  KABC::DistributionList *dlist = ab->createDistributionList( name );
+
+  for (int i = 0; i < mRecipientsList->topLevelItemCount(); ++i) {
+    DistributionListItem *item = static_cast<DistributionListItem *>(
+        mRecipientsList->topLevelItem( i ));
+    if ( item && item->checkState( 0 ) == Qt::Checked ) {
+      kDebug() << item->addressee().fullEmail() << item->addressee().uid();
       if ( item->isTransient() ) {
         ab->insertAddressee( item->addressee() );
       }
@@ -202,7 +218,6 @@ void DistributionListDialog::slotUser1()
         dlist->insertEntry( item->addressee(), item->email() );
       }
     }
-    i = i->nextSibling();
   }
 
   // FIXME: Ask the user which resource to save to instead of the default
@@ -215,9 +230,7 @@ void DistributionListDialog::slotUser1()
       ab->releaseSaveTicket( ticket );
 
   if ( saveError )
-    kdWarning(5006) << k_funcinfo << " Couldn't save new addresses in the distribution list just created to the address book" << endl;
-
-  manager.save();
+    kWarning(5006) <<" Couldn't save new addresses in the distribution list just created to the address book";
 
   close();
 }
