@@ -114,8 +114,7 @@ public:
   virtual bool noContent() const { return mNoContent; }
 
   /** Specify, that the folder can't contain mails. */
-  virtual void setNoContent(bool aNoContent)
-    { mNoContent = aNoContent; }
+  virtual void setNoContent(bool aNoContent);
 
   /** Returns, if the folder can't have children */
   virtual bool noChildren() const { return mNoChildren; }
@@ -278,7 +277,8 @@ public:
     If @p force is TRUE the files are closed regardless of reference count,
     and the reference count will be set to zero.
     @p owner is for debugging. */
-  virtual void close( const char *owner, bool force = false ) = 0;
+  virtual void close( const char *owner, bool force = false );
+  virtual void reallyDoClose() = 0;
 
   /** Try releasing @p folder if possible, something is attempting an exclusive access to it.
       Currently used for KMFolderSearch and the background tasks like expiry. */
@@ -354,6 +354,9 @@ public:
   /** Is the folder read-only? */
   virtual bool isReadOnly() const = 0;
 
+  /** Can messages in this folder be deleted? */
+  virtual bool canDeleteMessages() const;
+
   /** Returns the label of the folder for visualization. */
   QString label() const;
 
@@ -464,6 +467,9 @@ signals:
   /** Emitted when the readonly status of the folder changes. */
   void readOnlyChanged(KMFolder*);
 
+  /** Emitted when the no content state of the folder changes. */
+  void noContentChanged();
+
   /** Emitted before a message is removed from the folder. */
   void msgRemoved(KMFolder*, quint32 sernum);
 
@@ -606,9 +612,13 @@ friend class KMMsgDict;
   virtual void clearIndex(bool autoDelete=true, bool syncDict = false) = 0;
   virtual void truncateIndex() = 0;
 
-  virtual qint64 doFolderSize() const { return 0; };
+  /**
+   * Compute the size of the folder storage.
+   * Return -1 if the value is meaningless.
+   */
+  virtual qint64 doFolderSize() const { return -1; };
 
-  /** Returns full path to .index, .index.ids or .index.sorted file (depending on @a suffix). 
+  /** Returns full path to .index, .index.ids or .index.sorted file (depending on @a suffix).
    The former path (in case of mmap-based implementation) is returned when @a suffix is empty. */
   virtual QString location(const QString& suffix) const;
 
@@ -628,7 +638,7 @@ friend class KMMsgDict;
   /** number of unread messages, -1 if not yet set */
   int mUnreadMsgs, mGuessedUnreadMsgs;
   int mTotalMsgs;
-  qint64 mSize;
+  mutable qint64 mCachedSize;
   bool mWriteConfigEnabled :1;
   /** sven: true if on destruct folder needs to be compacted. */
   bool needsCompact :1;
