@@ -297,7 +297,7 @@ public:
 
   QWidget* mainWindow() { return mMainWindow; }
 
-  /** Returns wether the message should be decryted. */
+  /** Returns whether the message should be decryted. */
   bool decryptMessage() const;
 
   /** Enforce message decryption. */
@@ -311,6 +311,12 @@ public:
 
   KMail::CSSHelper* cssHelper() const;
 
+  /* show or hide the list that points to the attachments */
+  bool showAttachmentQuicklist() const { return mShowAttachmentQuicklist; }
+
+  /* show or hide the list that points to the attachments */
+  void setShowAttachmentQuicklist( bool showAttachmentQuicklist = true ) { mShowAttachmentQuicklist = showAttachmentQuicklist; }
+
 signals:
   /** Emitted after parsing of a message to have it stored
       in unencrypted state in it's folder. */
@@ -403,6 +409,7 @@ protected slots:
   void slotCycleHeaderStyles();
   void slotBriefHeaders();
   void slotFancyHeaders();
+  void slotEnterpriseHeaders();
   void slotStandardHeaders();
   void slotLongHeaders();
   void slotAllHeaders();
@@ -419,7 +426,7 @@ protected slots:
 
   /** Print message. Called on as a response of finished() signal of mPartHtmlWriter
       after rendering is finished.
-      In the very end it deletes the KMReaderWin window that was created 
+      In the very end it deletes the KMReaderWin window that was created
       for the purpose of rendering. */
   void slotPrintMsg();
 
@@ -447,7 +454,7 @@ protected:
 
   /** Creates a nice mail header depending on the current selected
     header style. */
-  QString writeMsgHeader(KMMessage* aMsg, bool hasVCard=false);
+  QString writeMsgHeader( KMMessage* aMsg, bool hasVCard = false, bool topLevel = false );
 
   /** Writes the given message part to a temporary file and returns the
       name of this file or QString() if writing failed.
@@ -479,6 +486,7 @@ protected:
 
 private slots:
   void slotSetEncoding();
+  void injectAttachments();
 
 private:
   void adjustLayout();
@@ -492,6 +500,8 @@ private:
   /** Read override codec from configuration */
   void readGlobalOverrideCodec();
 
+  QString renderAttachments( partNode *node, const QColor &bgColor );
+
 private:
   bool mHtmlMail, mHtmlLoadExternal, mHtmlOverride, mHtmlLoadExtOverride;
   int mAtmCurrent;
@@ -543,7 +553,7 @@ private:
   KToggleAction *mToggleFixFontAction;
   KUrl mUrlClicked;
   KMail::HtmlWriter * mHtmlWriter;
-  /** Used only to be able to connect and disconnect finished() signal 
+  /** Used only to be able to connect and disconnect finished() signal
       in printMsg() and slotPrintMsg() since mHtmlWriter points only to abstract non-QObject class. */
   QPointer<KMail::KHtmlPartHtmlWriter> mPartHtmlWriter;
 
@@ -555,6 +565,8 @@ private:
 	int mLevelQuote;
   bool mDecrytMessageOverwrite;
   bool mShowSignatureDetails;
+  bool mShowAttachmentQuicklist;
+  bool mExternalWindow;
 };
 
 
