@@ -1,7 +1,7 @@
 // -*- mode: C++; c-file-style: "gnu" -*-
 /***************************************************************************
                           kmsystemtray.cpp  -  description
-                             -------------------
+                             ------------------
     begin                : Fri Aug 31 22:38:44 EDT 2001
     copyright            : (C) 2001 by Ryan Breen
     email                : ryan@porivo.com
@@ -16,32 +16,32 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <config.h>
 
 #include "kmsystemtray.h"
 #include "kmfolder.h"
-#include "kmfoldertree.h"
 #include "kmfoldermgr.h"
 #include "kmfolderimap.h"
 #include "kmmainwidget.h"
 #include "accountmanager.h"
+
 using KMail::AccountManager;
 #include "globalsettings.h"
 
-#include <kapplication.h>
-#include <kmainwindow.h>
+#include "mainfolderview.h"
+
+#include <kxmlguiwindow.h>
 #include <kglobalsettings.h>
 #include <kiconloader.h>
-#include <kiconeffect.h>
-#include <kwin.h>
+#include <kcolorscheme.h>
+#include <kwindowsystem.h>
 #include <kdebug.h>
-#include <kpopupmenu.h>
+#include <KMenu>
 
-#include <qpainter.h>
-#include <qbitmap.h>
-#include <qtooltip.h>
-#include <qwidgetlist.h>
-#include <qobjectlist.h>
+#include <QPainter>
+#include <QBitmap>
+#include <QWidget>
+#include <QObject>
+#include <QSignalMapper>
 
 #include <math.h>
 #include <assert.h>
@@ -57,44 +57,47 @@ using KMail::AccountManager;
  * with its count of unread messages, allowing the user to jump
  * to the first unread message in each folder.
  */
-KMSystemTray::KMSystemTray(QWidget *parent, const char *name)
-  : KSystemTray( parent, name ),
+KMSystemTray::KMSystemTray(QWidget *parent)
+  : KSystemTrayIcon( parent),
     mParentVisible( true ),
     mPosOfMainWin( 0, 0 ),
     mDesktopOfMainWin( 0 ),
     mMode( GlobalSettings::EnumSystemTrayPolicy::ShowOnUnread ),
     mCount( 0 ),
-    mNewMessagePopupId(-1),
-    mPopupMenu(0)
+    mNewMessagesPopup( 0 ),
+    mSendQueued( 0 )
 {
-  setAlignment( AlignCenter );
-  kdDebug(5006) << "Initting systray" << endl;
+  kDebug(5006) <<"Initting systray";
 
   mLastUpdate = time( 0 );
-  mUpdateTimer = new QTimer( this, "systraytimer" );
+  mUpdateTimer = new QTimer( this );
+  mUpdateTimer->setSingleShot( true );
   connect( mUpdateTimer, SIGNAL( timeout() ), SLOT( updateNewMessages() ) );
 
-  mDefaultIcon = loadIcon( "kmail" );
-  mLightIconImage = loadIcon( "kmaillight" ).convertToImage();
-
-  setPixmap(mDefaultIcon);
-
+  mDefaultIcon = KIcon( "kmail" ).pixmap( 22 );
+  setIcon( mDefaultIcon );
+#ifdef Q_WS_X11
   KMMainWidget * mainWidget = kmkernel->getKMMainWidget();
   if ( mainWidget ) {
     QWidget * mainWin = mainWidget->topLevelWidget();
     if ( mainWin ) {
-      mDesktopOfMainWin = KWin::windowInfo( mainWin->winId(),
+      mDesktopOfMainWin = KWindowSystem::windowInfo( mainWin->winId(),
                                             NET::WMDesktop ).desktop();
       mPosOfMainWin = mainWin->pos();
     }
   }
-
+#endif
   // register the applet with the kernel
   kmkernel->registerSystemTrayApplet( this );
 
   /** Initiate connections between folders and this object */
   foldersChanged();
 
+  connect( this, SIGNAL( activated( QSystemTrayIcon::ActivationReason ) ),
+           this, SLOT( slotActivated( QSystemTrayIcon::ActivationReason ) ) );
+  connect( contextMenu(), SIGNAL( aboutToShow() ),
+           this, SLOT( slotContextMenuAboutToShow() ) );
+
   connect( kmkernel->folderMgr(), SIGNAL(changed()), SLOT(foldersChanged()));
   connect( kmkernel->imapFolderMgr(), SIGNAL(changed()), SLOT(foldersChanged()));
   connect( kmkernel->dimapFolderMgr(), SIGNAL(changed()), SLOT(foldersChanged()));
@@ -106,66 +109,65 @@ KMSystemTray::KMSystemTray(QWidget *parent, const char *name)
 
 void KMSystemTray::buildPopupMenu()
 {
-  // Delete any previously created popup menu
-  delete mPopupMenu;
-  mPopupMenu = 0;
+  if ( !contextMenu() ) {
+    setContextMenu( new KMenu() );
+  }
+
+  contextMenu()->clear();
 
-  mPopupMenu = new KPopupMenu();
   KMMainWidget * mainWidget = kmkernel->getKMMainWidget();
   if ( !mainWidget )
     return;
 
-  mPopupMenu->insertTitle(*(this->pixmap()), "KMail");
-  KAction * action;
+  static_cast<KMenu*>( contextMenu() )->addTitle(this->icon(), "KMail");
+  QAction * action;
   if ( ( action = mainWidget->action("check_mail") ) )
-    action->plug( mPopupMenu );
+    contextMenu()->addAction( action );
   if ( ( action = mainWidget->action("check_mail_in") ) )
-    action->plug( mPopupMenu );
-  if ( ( action = mainWidget->action("send_queued") ) )
-    action->plug( mPopupMenu );
+    contextMenu()->addAction( action );
+  if ( ( mSendQueued = mainWidget->action("send_queued") ) )
+    contextMenu()->addAction( mSendQueued );
   if ( ( action = mainWidget->action("send_queued_via") ) )
-    action->plug( mPopupMenu );
-  mPopupMenu->insertSeparator();
+    contextMenu()->addAction( action );
+  contextMenu()->addSeparator();
   if ( ( action = mainWidget->action("new_message") ) )
-    action->plug( mPopupMenu );
+    contextMenu()->addAction( action );
   if ( ( action = mainWidget->action("kmail_configure_kmail") ) )
-    action->plug( mPopupMenu );
-  mPopupMenu->insertSeparator();
+    contextMenu()->addAction( action );
+  contextMenu()->addSeparator();
 
-  KMainWindow *mainWin = ::qt_cast<KMainWindow*>(kmkernel->getKMMainWidget()->topLevelWidget());
+  KXmlGuiWindow *mainWin = ::qobject_cast<KXmlGuiWindow*>(kmkernel->getKMMainWidget()->topLevelWidget());
   if(mainWin)
     if ( ( action=mainWin->actionCollection()->action("file_quit") ) )
-      action->plug( mPopupMenu );
+      contextMenu()->addAction( action );
 }
 
 KMSystemTray::~KMSystemTray()
 {
   // unregister the applet
   kmkernel->unregisterSystemTrayApplet( this );
-
-  delete mPopupMenu;
-  mPopupMenu = 0;
 }
 
 void KMSystemTray::setMode(int newMode)
 {
   if(newMode == mMode) return;
 
-  kdDebug(5006) << "Setting systray mMode to " << newMode << endl;
+  kDebug(5006) <<"Setting systray mMode to" << newMode;
   mMode = newMode;
 
   switch ( mMode ) {
   case GlobalSettings::EnumSystemTrayPolicy::ShowAlways:
-    if ( isHidden() )
+    if ( !isVisible() )
       show();
     break;
   case GlobalSettings::EnumSystemTrayPolicy::ShowOnUnread:
-    if ( mCount == 0 && !isHidden() )
+    if ( mCount == 0 && isVisible() )
       hide();
-    else if ( mCount > 0 && isHidden() )
+    else if ( mCount > 0 && !isVisible() )
       show();
+    break;
   default:
-    kdDebug(5006) << k_funcinfo << " Unknown systray mode " << mMode << endl;
+    kDebug(5006) <<" Unknown systray mode" << mMode;
   }
 }
 
@@ -183,8 +185,7 @@ void KMSystemTray::updateCount()
 {
   if(mCount != 0)
   {
-    int oldPixmapWidth = pixmap()->size().width();
-    int oldPixmapHeight = pixmap()->size().height();
+    int oldPixmapWidth = mDefaultIcon.size().width();
 
     QString countString = QString::number( mCount );
     QFont countFont = KGlobalSettings::generalFont();
@@ -192,51 +193,41 @@ void KMSystemTray::updateCount()
 
     // decrease the size of the font for the number of unread messages if the
     // number doesn't fit into the available space
-    float countFontSize = countFont.pointSizeFloat();
+    float countFontSize = countFont.pointSizeF();
     QFontMetrics qfm( countFont );
     int width = qfm.width( countString );
-    if( width > oldPixmapWidth )
+    if( width > (oldPixmapWidth - 2) )
     {
-      countFontSize *= float( oldPixmapWidth ) / float( width );
-      countFont.setPointSizeFloat( countFontSize );
+      countFontSize *= float( oldPixmapWidth - 2 ) / float( width );
+      countFont.setPointSizeF( countFontSize );
     }
 
-    // Create an image which represents the number of unread messages
-    // and which has a transparent background.
-    // Unfortunately this required the following twisted code because for some
-    // reason text that is drawn on a transparent pixmap is invisible
-    // (apparently the alpha channel isn't changed when the text is drawn).
-    // Therefore I have to draw the text on a solid background and then remove
-    // the background by making it transparent with QPixmap::setMask. This
-    // involves the slow createHeuristicMask() function (from the API docs:
-    // "This function is slow because it involves transformation to a QImage,
-    // non-trivial computations and a transformation back to a QBitmap."). Then
-    // I have to convert the resulting QPixmap to a QImage in order to overlay
-    // the light KMail icon with the number (because KIconEffect::overlay only
-    // works with QImage). Finally the resulting QImage has to be converted
-    // back to a QPixmap.
-    // That's a lot of work for overlaying the KMail icon with the number of
-    // unread messages, but every other approach I tried failed miserably.
-    //                                                           IK, 2003-09-22
-    QPixmap numberPixmap( oldPixmapWidth, oldPixmapHeight );
-    numberPixmap.fill( Qt::white );
-    QPainter p( &numberPixmap );
-    p.setFont( countFont );
-    p.setPen( Qt::blue );
-    p.drawText( numberPixmap.rect(), Qt::AlignCenter, countString );
-    numberPixmap.setMask( numberPixmap.createHeuristicMask() );
-    QImage numberImage = numberPixmap.convertToImage();
-
     // Overlay the light KMail icon with the number image
-    QImage iconWithNumberImage = mLightIconImage.copy();
-    KIconEffect::overlay( iconWithNumberImage, numberImage );
+    QPixmap iconWithNumberImage = mDefaultIcon;
+    QPainter p( &iconWithNumberImage );
+    p.setFont( countFont );
+    KColorScheme scheme( QPalette::Active, KColorScheme::View );
 
-    QPixmap iconWithNumber;
-    iconWithNumber.convertFromImage( iconWithNumberImage );
-    setPixmap( iconWithNumber );
+    qfm = QFontMetrics( countFont );
+    QRect boundingRect = qfm.tightBoundingRect( countString );
+    boundingRect.adjust( 0, 0, 0, 2 );
+    boundingRect.setHeight( qMin( boundingRect.height(), oldPixmapWidth ) );
+    boundingRect.moveTo( (oldPixmapWidth - boundingRect.width()) / 2,
+                         ((oldPixmapWidth - boundingRect.height()) / 2) - 1 );
+    p.setOpacity( 0.7 );
+    p.setBrush( scheme.background( KColorScheme::LinkBackground ) );
+    p.setPen( scheme.background( KColorScheme::LinkBackground ).color() );
+    p.drawRoundedRect( boundingRect, 2.0, 2.0 );
+
+    p.setBrush( Qt::NoBrush );
+    p.setPen( scheme.foreground( KColorScheme::LinkText ).color() );
+    p.setOpacity( 1.0 );
+    p.drawText( iconWithNumberImage.rect(), Qt::AlignCenter, countString );
+
+    setIcon( iconWithNumberImage );
   } else
   {
-    setPixmap( mDefaultIcon );
+    setIcon( mDefaultIcon );
   }
 }
 
@@ -251,6 +242,7 @@ void KMSystemTray::foldersChanged()
    * unread message was in a folder that was just removed.
    */
   mFoldersWithUnread.clear();
+  mPendingUpdates.clear();
   mCount = 0;
 
   if ( mMode == GlobalSettings::EnumSystemTrayPolicy::ShowOnUnread ) {
@@ -261,7 +253,7 @@ void KMSystemTray::foldersChanged()
   disconnect(this, SLOT(updateNewMessageNotification(KMFolder *)));
 
   QStringList folderNames;
-  QValueList<QGuardedPtr<KMFolder> > folderList;
+  QList<QPointer<KMFolder> > folderList;
   kmkernel->folderMgr()->createFolderList(&folderNames, &folderList);
   kmkernel->imapFolderMgr()->createFolderList(&folderNames, &folderList);
   kmkernel->dimapFolderMgr()->createFolderList(&folderNames, &folderList);
@@ -269,13 +261,13 @@ void KMSystemTray::foldersChanged()
 
   QStringList::iterator strIt = folderNames.begin();
 
-  for(QValueList<QGuardedPtr<KMFolder> >::iterator it = folderList.begin();
+  for(QList<QPointer<KMFolder> >::iterator it = folderList.begin();
      it != folderList.end() && strIt != folderNames.end(); ++it, ++strIt)
   {
     KMFolder * currentFolder = *it;
     QString currentName = *strIt;
 
-    if ( ((!currentFolder->isSystemFolder() || (currentFolder->name().lower() == "inbox")) ||
+    if ( ((!currentFolder->isSystemFolder() || (currentFolder->name().toLower() == "inbox")) ||
          (currentFolder->folderType() == KMFolderTypeImap)) &&
          !currentFolder->ignoreNewMail() )
     {
@@ -286,6 +278,10 @@ void KMSystemTray::foldersChanged()
       /** Check all new folders to see if we started with any new messages */
       updateNewMessageNotification(currentFolder);
     }
+    else {
+      disconnect( currentFolder, SIGNAL( numUnreadMsgsChanged(KMFolder*) ),
+                  this, SLOT( updateNewMessageNotification(KMFolder *) ) );
+    }
   }
 }
 
@@ -293,55 +289,58 @@ void KMSystemTray::foldersChanged()
  * On left mouse click, switch focus to the first KMMainWidget.  On right
  * click, bring up a list of all folders with a count of unread messages.
  */
-void KMSystemTray::mousePressEvent(QMouseEvent *e)
+void KMSystemTray::slotActivated( QSystemTrayIcon::ActivationReason reason )
 {
+  kDebug(5006) <<"trigger:" << reason;
+
   // switch to kmail on left mouse button
-  if( e->button() == LeftButton )
+  if( reason == QSystemTrayIcon::Trigger )
   {
     if( mParentVisible && mainWindowIsOnCurrentDesktop() )
       hideKMail();
     else
       showKMail();
   }
+}
 
-  // open popup menu on right mouse button
-  if( e->button() == RightButton )
-  {
-    mPopupFolders.clear();
-    mPopupFolders.reserve( mFoldersWithUnread.count() );
+void KMSystemTray::slotContextMenuAboutToShow() 
+{
+  // Rebuild popup menu before show to minimize race condition if
+  // the base KMainWidget is closed.
+  buildPopupMenu();
 
-    // Rebuild popup menu at click time to minimize race condition if
-    // the base KMainWidget is closed.
-    buildPopupMenu();
-
-    if(mNewMessagePopupId != -1)
-    {
-      mPopupMenu->removeItem(mNewMessagePopupId);
-    }
-
-    if(mFoldersWithUnread.count() > 0)
-    {
-      KPopupMenu *newMessagesPopup = new KPopupMenu();
-
-      QMap<QGuardedPtr<KMFolder>, int>::Iterator it = mFoldersWithUnread.begin();
-      for(uint i=0; it != mFoldersWithUnread.end(); ++i)
-      {
-        kdDebug(5006) << "Adding folder" << endl;
-        mPopupFolders.append( it.key() );
-        QString item = prettyName(it.key()) + " (" + QString::number(it.data()) + ")";
-        newMessagesPopup->insertItem(item, this, SLOT(selectedAccount(int)), 0, i);
-        ++it;
-      }
-
-      mNewMessagePopupId = mPopupMenu->insertItem(i18n("New Messages In"),
-                                                  newMessagesPopup, mNewMessagePopupId, 3);
-
-      kdDebug(5006) << "Folders added" << endl;
-    }
-
-    mPopupMenu->popup(e->globalPos());
+  if ( mNewMessagesPopup != 0 ) {
+    contextMenu()->removeAction( mNewMessagesPopup->menuAction() );
+    delete mNewMessagesPopup;
+    mNewMessagesPopup = 0;
   }
 
+  if ( mFoldersWithUnread.count() > 0 )
+  {
+    mNewMessagesPopup = new KMenu();
+
+    QMap<QPointer<KMFolder>, int>::Iterator it = mFoldersWithUnread.begin();
+    QSignalMapper *folderMapper = new QSignalMapper( this );
+    connect( folderMapper, SIGNAL( mapped( int ) ),
+             this, SLOT( selectedAccount( int ) ) );
+
+    for(uint i=0; it != mFoldersWithUnread.end(); ++i)
+    {
+      mPopupFolders.append( it.key() );
+      QString folderText = prettyName(it.key()) + " (" + QString::number(it.value()) + ')';
+      QAction *action = new QAction( folderText, this );
+      connect( action, SIGNAL( triggered( bool ) ),
+               folderMapper, SLOT( map() ) );
+      folderMapper->setMapping( action, i );
+      mNewMessagesPopup->addAction( action );
+      ++it;
+    }
+
+    mNewMessagesPopup->setTitle( i18n("New Messages In") );
+    contextMenu()->insertAction( mSendQueued, mNewMessagesPopup->menuAction() );
+
+    kDebug(5006) <<"Folders added";
+  }
 }
 
 /**
@@ -359,12 +358,12 @@ QString KMSystemTray::prettyName(KMFolder * fldr)
     if((imap->account() != 0) &&
        (imap->account()->name() != 0) )
     {
-      kdDebug(5006) << "IMAP folder, prepend label with type" << endl;
+      kDebug(5006) <<"IMAP folder, prepend label with type";
       rvalue = imap->account()->name() + "->" + rvalue;
     }
   }
 
-  kdDebug(5006) << "Got label " << rvalue << endl;
+  kDebug(5006) <<"Got label" << rvalue;
 
   return rvalue;
 }
@@ -372,6 +371,7 @@ QString KMSystemTray::prettyName(KMFolder * fldr)
 
 bool KMSystemTray::mainWindowIsOnCurrentDesktop()
 {
+#ifdef Q_WS_X11
   KMMainWidget * mainWidget = kmkernel->getKMMainWidget();
   if ( !mainWidget )
     return false;
@@ -380,8 +380,11 @@ bool KMSystemTray::mainWindowIsOnCurrentDesktop()
   if ( !mainWin )
     return false;
 
-  return KWin::windowInfo( mainWin->winId(),
+  return KWindowSystem::windowInfo( mainWin->winId(),
                            NET::WMDesktop ).isOnCurrentDesktop();
+#else
+  return true;
+#endif
 }
 
 /**
@@ -394,20 +397,24 @@ void KMSystemTray::showKMail()
     return;
   QWidget *mainWin = kmkernel->getKMMainWidget()->topLevelWidget();
   assert(mainWin);
+#ifdef Q_WS_X11
   if(mainWin)
   {
-    KWin::WindowInfo cur =  KWin::windowInfo( mainWin->winId(), NET::WMDesktop );
+    KWindowInfo cur = KWindowSystem::windowInfo( mainWin->winId(), NET::WMDesktop );
     if ( cur.valid() ) mDesktopOfMainWin = cur.desktop();
     // switch to appropriate desktop
     if ( mDesktopOfMainWin != NET::OnAllDesktops )
-      KWin::setCurrentDesktop( mDesktopOfMainWin );
+      KWindowSystem::setCurrentDesktop( mDesktopOfMainWin );
     if ( !mParentVisible ) {
       if ( mDesktopOfMainWin == NET::OnAllDesktops )
-        KWin::setOnAllDesktops( mainWin->winId(), true );
-      mainWin->move( mPosOfMainWin );
-      mainWin->show();
+        KWindowSystem::setOnAllDesktops( mainWin->winId(), true );
     }
-    KWin::activateWindow( mainWin->winId() );
+    KWindowSystem::activateWindow( mainWin->winId() );
+  }
+#endif
+  if ( !mParentVisible ) {
+    mainWin->move( mPosOfMainWin );
+    mainWin->show();
     mParentVisible = true;
   }
   kmkernel->raise();
@@ -424,11 +431,13 @@ void KMSystemTray::hideKMail()
   assert(mainWin);
   if(mainWin)
   {
-    mDesktopOfMainWin = KWin::windowInfo( mainWin->winId(),
-                                          NET::WMDesktop ).desktop();
     mPosOfMainWin = mainWin->pos();
+#ifdef Q_WS_X11
+    mDesktopOfMainWin = KWindowSystem::windowInfo( mainWin->winId(),
+                                          NET::WMDesktop ).desktop();
     // iconifying is unnecessary, but it looks cooler
-    KWin::iconifyWindow( mainWin->winId() );
+    KWindowSystem::minimizeWindow( mainWin->winId() );
+#endif
     mainWin->hide();
     mParentVisible = false;
   }
@@ -447,7 +456,7 @@ void KMSystemTray::updateNewMessageNotification(KMFolder * fldr)
   if( !fldr ||
       fldr->folderType() == KMFolderTypeSearch )
   {
-    // kdDebug(5006) << "Null or a search folder, can't mess with that" << endl;
+    // kDebug(5006) <<"Null or a search folder, can't mess with that";
     return;
   }
 
@@ -457,96 +466,88 @@ void KMSystemTray::updateNewMessageNotification(KMFolder * fldr)
     updateNewMessages();
   }
   else {
-    mUpdateTimer->start(150, true);
+    mUpdateTimer->start(150);
   }
 }
 
 void KMSystemTray::updateNewMessages()
 {
-  for ( QMap<QGuardedPtr<KMFolder>, bool>::Iterator it = mPendingUpdates.begin();
-        it != mPendingUpdates.end(); ++it)
+  for ( QMap<QPointer<KMFolder>, bool>::Iterator it1 = mPendingUpdates.begin();
+        it1 != mPendingUpdates.end(); ++it1)
   {
-  KMFolder *fldr = it.key();
-  if ( !fldr ) // deleted folder
-    continue;
+    KMFolder *fldr = it1.key();
+    if ( !fldr ) // deleted folder
+      continue;
 
-  /** The number of unread messages in that folder */
-  int unread = fldr->countUnread();
+    // The number of unread messages in that folder
+    int unread = fldr->countUnread();
 
-  QMap<QGuardedPtr<KMFolder>, int>::Iterator it =
-      mFoldersWithUnread.find(fldr);
-  bool unmapped = (it == mFoldersWithUnread.end());
+    QMap<QPointer<KMFolder>, int>::Iterator it = mFoldersWithUnread.find( fldr );
+    bool unmapped = ( it == mFoldersWithUnread.end() );
 
-  /** If the folder is not mapped yet, increment count by numUnread
-      in folder */
-  if(unmapped) mCount += unread;
-  /* Otherwise, get the difference between the numUnread in the folder and
-   * our last known version, and adjust mCount with that difference */
-  else
-  {
-    int diff = unread - it.data();
-    mCount += diff;
-  }
+    // If the folder is not mapped yet, increment count by numUnread
+    // in folder
+    if ( unmapped )
+      mCount += unread;
 
-  if(unread > 0)
-  {
-    /** Add folder to our internal store, or update unread count if already mapped */
-    mFoldersWithUnread.insert(fldr, unread);
-    //kdDebug(5006) << "There are now " << mFoldersWithUnread.count() << " folders with unread" << endl;
-  }
-
-  /**
-   * Look for folder in the list of folders already represented.  If there are
-   * unread messages and the system tray icon is hidden, show it.  If there are
-   * no unread messages, remove the folder from the mapping.
-   */
-  if(unmapped)
-  {
-    /** Spurious notification, ignore */
-    if(unread == 0) continue;
-
-    /** Make sure the icon will be displayed */
-    if ( ( mMode == GlobalSettings::EnumSystemTrayPolicy::ShowOnUnread )
-         && isHidden() ) {
-      show();
+    //Otherwise, get the difference between the numUnread in the folder and
+    // our last known version, and adjust mCount with that difference
+    else {
+      int diff = unread - it.value();
+      mCount += diff;
     }
 
-  } else
-  {
+    if ( unread > 0 ) {
+      // Add folder to our internal store, or update unread count if already mapped
+      mFoldersWithUnread.insert( fldr, unread );
+      //kDebug(5006) <<"There are now" << mFoldersWithUnread.count() <<" folders with unread";
+    }
 
-    if(unread == 0)
-    {
-      kdDebug(5006) << "Removing folder from internal store " << fldr->name() << endl;
+    /*
+     * Look for folder in the list of folders already represented.  If there are
+     * unread messages and the system tray icon is hidden, show it.  If there are
+     * no unread messages, remove the folder from the mapping.
+     */
+    if ( unmapped ) {
 
-      /** Remove the folder from the internal store */
-      mFoldersWithUnread.remove(fldr);
+      // Spurious notification, ignore
+      if ( unread == 0 )
+        continue;
 
-      /** if this was the last folder in the dictionary, hide the systemtray icon */
-      if(mFoldersWithUnread.count() == 0)
-      {
-        mPopupFolders.clear();
-        disconnect(this, SLOT(selectedAccount(int)));
+      // Make sure the icon will be displayed
+      if ( ( mMode == GlobalSettings::EnumSystemTrayPolicy::ShowOnUnread ) &&
+           !isVisible() ) {
+        show();
+      }
+    }
+    else {
+      if ( unread == 0 ) {
+        kDebug(5006) << "Removing folder from internal store" << fldr->name();
 
-        mCount = 0;
+        // Remove the folder from the internal store
+        mFoldersWithUnread.remove(fldr);
 
-        if ( mMode == GlobalSettings::EnumSystemTrayPolicy::ShowOnUnread ) {
-          hide();
+        // if this was the last folder in the dictionary, hide the systemtray icon
+        if ( mFoldersWithUnread.count() == 0 ) {
+          mPopupFolders.clear();
+          disconnect ( this, SLOT( selectedAccount( int ) ) );
+          mCount = 0;
+
+          if ( mMode == GlobalSettings::EnumSystemTrayPolicy::ShowOnUnread ) {
+            hide();
+          }
         }
       }
     }
   }
-
-  }
   mPendingUpdates.clear();
   updateCount();
 
-  /** Update tooltip to reflect count of unread messages */
-  QToolTip::remove(this);
-  QToolTip::add(this, mCount == 0 ?
-		      i18n("There are no unread messages")
-		      : i18n("There is 1 unread message.",
-                             "There are %n unread messages.",
-                           mCount));
+  // Update tooltip to reflect count of unread messages
+  setToolTip( mCount == 0 ? i18n("KMail - There are no unread messages")
+                          : i18np("KMail - 1 unread message",
+                                  "KMail - %1 unread messages",
+                                  mCount));
 
   mLastUpdate = time( 0 );
 }
@@ -572,13 +573,15 @@ void KMSystemTray::selectedAccount(int id)
   /** Select folder */
   KMFolder * fldr = mPopupFolders.at(id);
   if(!fldr) return;
-  KMFolderTree * ft = mainWidget->folderTree();
-  if(!ft) return;
-  QListViewItem * fldrIdx = ft->indexOfFolder(fldr);
-  if(!fldrIdx) return;
 
-  ft->setCurrentItem(fldrIdx);
-  ft->selectCurrentFolder();
+  KMail::MainFolderView * ftw = mainWidget->mainFolderView();
+  if ( !ftw ) return;
+  ftw->setCurrentFolder( fldr );
+}
+
+bool KMSystemTray::hasUnreadMail() const
+{
+  return ( mCount != 0 );
 }
 
 #include "kmsystemtray.moc"
