@@ -84,7 +84,7 @@ using namespace KCal;
 KNote::KNote( const QDomDocument& buildDoc, Journal *j, QWidget *parent )
   : QFrame( parent, Qt::FramelessWindowHint ), m_label( 0 ), m_grip( 0 ),
     m_button( 0 ), m_tool( 0 ), m_editor( 0 ), m_config( 0 ), m_journal( j ),
-    m_find( 0 ), m_kwinConf( KSharedConfig::openConfig( "kwinrc" ) ), m_blockEmitDataChanged( false )
+    m_find( 0 ), m_kwinConf( KSharedConfig::openConfig( "kwinrc" ) ), m_blockEmitDataChanged( false ),mBlockWriteConfigDuringCommitData( false )
 {
   setAcceptDrops( true );
   setAttribute( Qt::WA_DeleteOnClose );
@@ -332,6 +332,12 @@ void KNote::slotUpdateReadOnly()
   updateFocus();
 }
 
+
+void KNote::commitData()
+{
+  mBlockWriteConfigDuringCommitData = true;
+}
+
 void KNote::slotClose()
 {
 #ifdef Q_WS_X11
@@ -344,9 +350,12 @@ void KNote::slotClose()
 #endif
 
   m_editor->clearFocus();
-  m_config->setHideNote( true );
-  m_config->setPosition( pos() );
-  m_config->writeConfig();
+  if( !mBlockWriteConfigDuringCommitData )
+  {
+    m_config->setHideNote( true );
+    m_config->setPosition( pos() );
+    m_config->writeConfig();
+  }
   // just hide the note so it's still available from the dock window
   hide();
 }
@@ -1039,25 +1048,36 @@ void KNote::updateLabelAlignment()
 
 void KNote::updateFocus()
 {
-  if ( hasFocus() ) {
+  if ( hasFocus() )
+  {
     m_button->show();
 
-    if ( !m_editor->isReadOnly() ) {
-      if ( m_tool && m_tool->isHidden() && m_editor->acceptRichText() ) {
-        m_tool->show();
-	m_grip->show();
-        setGeometry( x(), y(), width(), height() + m_tool->height() );
-      }
-    } else if ( m_tool && !m_tool->isHidden() ) {
-      m_tool->hide();
-      setGeometry( x(), y(), width(), height() - m_tool->height() );
-      updateLayout();     // to update the minimum height
+    if ( !m_editor->isReadOnly() )
+    {
+        if ( m_tool && m_tool->isHidden() && m_editor->acceptRichText() )
+        {
+            m_tool->show();
+            setGeometry( x(), y(), width(), height() + m_tool->height() );
+        }
+        m_grip->show();
     }
-  } else {
+    else
+    {
+        if ( m_tool && !m_tool->isHidden() ) {
+            m_tool->hide();
+            setGeometry( x(), y(), width(), height() - m_tool->height() );
+            updateLayout();     // to update the minimum height
+        }
+        m_grip->hide();
+    }
+  }
+  else
+  {
     m_button->hide();
     m_grip->hide();
 
-    if ( m_tool && !m_tool->isHidden() ) {
+    if ( m_tool && !m_tool->isHidden() )
+    {
       m_tool->hide();
       setGeometry( x(), y(), width(), height() - m_tool->height() );
       updateLayout();     // to update the minimum height
