@@ -2,7 +2,7 @@
   This file is part of Kontact.
 
   Copyright (c) 2003 Tobias Koenig <tokoe@kde.org>
-  Copyright (c) 2005-2006,2008 Allen Winter <winter@kde.org>
+  Copyright (c) 2005-2006,2008-2009 Allen Winter <winter@kde.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -74,7 +74,9 @@ TodoSummaryWidget::TodoSummaryWidget( TodoPlugin *plugin, QWidget *parent )
   mLayout->setRowStretch( 6, 1 );
 
   mCalendar = KOrg::StdCalendar::self();
-  mCalendar->load();
+  
+  //If the kpart isn't created yet, it's created now, and mCalendar is loaded
+  mPlugin->part();
 
   connect( mCalendar, SIGNAL(calendarChanged()), SLOT(updateView()) );
   connect( mPlugin->core(), SIGNAL(dayChanged(const QDate&)), SLOT(updateView()) );
@@ -254,6 +256,7 @@ void TodoSummaryWidget::updateView()
       urlLabel->setUrl( todo->uid() );
       urlLabel->installEventFilter( this );
       urlLabel->setTextFormat( Qt::RichText );
+      urlLabel->setWordWrap( true );
       mLayout->addWidget( urlLabel, counter, 4 );
       mLabels.append( urlLabel );
 
@@ -262,7 +265,8 @@ void TodoSummaryWidget::updateView()
       connect( urlLabel, SIGNAL(rightClickedUrl(const QString&)),
                this, SLOT(popupMenu(const QString&)) );
 
-      QString tipText( KCal::IncidenceFormatter::toolTipString( todo, true ) );
+      QString tipText( KCal::IncidenceFormatter::toolTipStr(
+                         todo, true, KPIM::KPimPrefs::timeSpec() ) );
       if ( !tipText.isEmpty() ) {
         urlLabel->setToolTip( tipText );
       }
@@ -284,11 +288,11 @@ void TodoSummaryWidget::updateView()
              "No pending to-dos due within the next %1 days",
              mDaysToGo ), this );
     noTodos->setAlignment( Qt::AlignHCenter | Qt::AlignVCenter );
-    mLayout->addWidget( noTodos, 0, 2 );
+    mLayout->addWidget( noTodos, 0, 0 );
     mLabels.append( noTodos );
   }
 
-  Q_FOREACH( label, mLabels ) {
+  Q_FOREACH( label, mLabels ) { //krazy:exclude=foreach as label is a pointer
     label->show();
   }
 }
