@@ -24,21 +24,20 @@
 */
 
 #include "koviewmanager.h"
+#include "koglobals.h"
+#include "koprefs.h"
 #include "calendarview.h"
 #include "datenavigator.h"
-#include "koagendaview.h"
-#include "views/oldmonthview/komonthview.h"
-#include "kolistview.h"
-#include "kowhatsnextview.h"
-#include "kojournalview.h"
-#include "kotimelineview.h"
-#include "kotimespentview.h"
-#include "koprefs.h"
-#include "koglobals.h"
 #include "navigatorbar.h"
-#include "multiagendaview.h"
+#include "views/agendaview/koagendaview.h"
+#include "views/listview/kolistview.h"
+#include "views/journalview/kojournalview.h"
 #include "views/monthview/monthview.h"
+#include "views/multiagendaview/multiagendaview.h"
 #include "views/todoview/kotodoview.h"
+#include "views/whatsnextview/kowhatsnextview.h"
+#include "views/timelineview/kotimelineview.h"
+#include "views/timespentview/kotimespentview.h"
 
 #include <KConfig>
 #include <KGlobal>
@@ -57,7 +56,6 @@ KOViewManager::KOViewManager( CalendarView *mainView )
   mTodoView = 0;
   mAgendaView = 0;
   mAgendaSideBySideView = 0;
-  mOldMonthView = 0;
   mListView = 0;
   mJournalView = 0;
   mTimelineView = 0;
@@ -83,7 +81,8 @@ void KOViewManager::readSettings( KConfig *config )
   if ( view == QLatin1String( "WhatsNext" ) ) {
     showWhatsNextView();
   } else if ( view == QLatin1String( "OldMonth" ) ) {
-    showOldMonthView();
+    // the oldmonth view is gone, so we assume the new month view
+    showMonthView();
   } else if ( view == QLatin1String( "List" ) ) {
     showListView();
   } else if ( view == QLatin1String( "Journal" ) ) {
@@ -108,8 +107,6 @@ void KOViewManager::writeSettings( KConfig *config )
   QString view;
   if ( mCurrentView == mWhatsNextView ) {
     view = "WhatsNext";
-  } else if ( mCurrentView == mOldMonthView ) {
-    view = "OldMonth";
   } else if ( mCurrentView == mListView ) {
     view = "List";
   } else if ( mCurrentView == mJournalView ) {
@@ -163,8 +160,7 @@ void KOViewManager::showView( KOrg::BaseView *view )
 
 void KOViewManager::raiseCurrentView()
 {
-  if ( ( mMonthView && KOPrefs::instance()->mFullViewMonth && mCurrentView == mMonthView ) ||
-       ( mTodoView && KOPrefs::instance()->mFullViewTodo && mCurrentView == mTodoView ) ) {
+  if ( mCurrentView && mCurrentView->usesFullWindow() ) {
     mMainView->showLeftFrame( false );
     if ( mCurrentView == mTodoView ) {
       mMainView->navigatorBar()->hide();
@@ -175,12 +171,7 @@ void KOViewManager::raiseCurrentView()
     mMainView->showLeftFrame( true );
     mMainView->navigatorBar()->hide();
   }
-  if ( !mAgendaViewTabs ||
-       ( mCurrentView != mAgendaView && mCurrentView != mAgendaSideBySideView ) ) {
-    mMainView->viewStack()->setCurrentWidget( mCurrentView );
-  } else {
-    mMainView->viewStack()->setCurrentWidget( mAgendaViewTabs );
-  }
+  mMainView->viewStack()->setCurrentWidget( widgetForView( mCurrentView  ) );
 }
 
 void KOViewManager::updateView()
@@ -366,46 +357,58 @@ void KOViewManager::showAgendaView()
     showBoth || KOPrefs::instance()->agendaViewCalendarDisplay() == KOPrefs::CalendarsSideBySide;
 
   QWidget *parent = mMainView->viewStack();
-  if ( !mAgendaViewTabs && showBoth ) {
-    mAgendaViewTabs = new KTabWidget( mMainView->viewStack() );
-    connect( mAgendaViewTabs, SIGNAL(currentChanged(QWidget *)),
-             this, SLOT(currentAgendaViewTabChanged(QWidget *)) );
+  if ( showBoth ) {
+    if ( !mAgendaViewTabs && showBoth ) {
+      mAgendaViewTabs = new KTabWidget( mMainView->viewStack() );
+      connect( mAgendaViewTabs, SIGNAL(currentChanged(QWidget *)),
+              this, SLOT(currentAgendaViewTabChanged(QWidget *)) );
+      mMainView->viewStack()->addWidget( mAgendaViewTabs );
+    }
     parent = mAgendaViewTabs;
-    mMainView->viewStack()->addWidget( mAgendaViewTabs );
   }
 
-  if ( !mAgendaView && showMerged ) {
-    mAgendaView = new KOAgendaView( mMainView->calendar(), parent );
-    mAgendaView->setObjectName( "KOViewManager::AgendaView" );
+  if ( showMerged ) {
+    if ( !mAgendaView ) {
+      mAgendaView = new KOAgendaView( mMainView->calendar(), parent );
+      mAgendaView->setObjectName( "KOViewManager::AgendaView" );
 
-    addView( mAgendaView, mAgendaViewTabs );
+      addView( mAgendaView, showBoth );
 
-    connect( mAgendaView,SIGNAL(zoomViewHorizontally(const QDate &,int)),
-             mMainView->dateNavigator(), SLOT(selectDates(const QDate &,int)) );
-    mAgendaView->readSettings();
-    if ( mAgendaViewTabs ) {
+      connect( mAgendaView,SIGNAL(zoomViewHorizontally(const QDate &,int)),
+               mMainView->dateNavigator(), SLOT(selectDates(const QDate &,int)) );
+      mAgendaView->readSettings();
+    }
+    if ( showBoth && mAgendaViewTabs->indexOf( mAgendaView ) < 0 ) {
       mAgendaViewTabs->addTab( mAgendaView, i18n( "Merged calendar" ) );
+    } else if ( !showBoth && mMainView->viewStack()->indexOf( mAgendaView ) < 0 ) {
+      mAgendaView->setParent( parent );
+      mMainView->viewStack()->addWidget( mAgendaView );
     }
   }
 
-  if ( !mAgendaSideBySideView && showSideBySide ) {
-    mAgendaSideBySideView = new MultiAgendaView( mMainView->calendar(), parent );
-    mAgendaSideBySideView->setObjectName( "KOViewManager::AgendaSideBySideView" );
-    addView( mAgendaSideBySideView, mAgendaViewTabs );
+  if ( showSideBySide ) {
+    if ( !mAgendaSideBySideView ) {
+      mAgendaSideBySideView = new MultiAgendaView( mMainView->calendar(), parent );
+      mAgendaSideBySideView->setObjectName( "KOViewManager::AgendaSideBySideView" );
+      addView( mAgendaSideBySideView, showBoth );
 
 /*
     connect( mAgendaSideBySideView,SIGNAL(zoomViewHorizontally(const QDate &,int)),
              mMainView->dateNavigator(),SLOT(selectDates(const QDate &,int)) );*/
-    if ( mAgendaViewTabs ) {
+    }
+    if ( showBoth && mAgendaViewTabs->indexOf( mAgendaSideBySideView ) < 0 ) {
       mAgendaViewTabs->addTab( mAgendaSideBySideView, i18n( "Calendars Side by Side" ) );
+    } else if ( !showBoth && mMainView->viewStack()->indexOf( mAgendaSideBySideView ) < 0 ) {
+      mAgendaSideBySideView->setParent( parent );
+      mMainView->viewStack()->addWidget( mAgendaSideBySideView );
     }
   }
 
-  if ( mAgendaViewTabs ) {
+  if ( showBoth ) {
     showView( static_cast<KOrg::BaseView*>( mAgendaViewTabs->currentWidget() ) );
-  } else if ( mAgendaView ) {
+  } else if ( showMerged ) {
     showView( mAgendaView );
-  } else if ( mAgendaSideBySideView ) {
+  } else if ( showSideBySide ) {
     showView( mAgendaSideBySideView );
   }
 }
@@ -419,9 +422,13 @@ void KOViewManager::showDayView()
 
 void KOViewManager::showWorkWeekView()
 {
-  QDate date = mMainView->activeDate();
-  showAgendaView();
-  mMainView->dateNavigator()->selectWorkWeek( date );
+  if ( KOGlobals::self()->getWorkWeekMask() != 0 ) {
+    QDate date = mMainView->activeDate();
+    showAgendaView();
+    mMainView->dateNavigator()->selectWorkWeek( date );
+  } else {
+    // TODO: tell the user he must configure work days
+  }
 }
 
 void KOViewManager::showWeekView()
@@ -438,17 +445,6 @@ void KOViewManager::showNextXView()
                                            KOPrefs::instance()->mNextXDays );
 }
 
-void KOViewManager::showOldMonthView()
-{
-  if ( !mOldMonthView ) {
-    mOldMonthView = new KOMonthView( mMainView->calendar(), mMainView->viewStack() );
-    mMonthView->setObjectName( "KOViewManager::OldMonthView" );
-    addView( mOldMonthView );
-  }
-
-  showView( mOldMonthView );
-}
-
 void KOViewManager::showTodoView()
 {
   if ( !mTodoView ) {
@@ -459,7 +455,7 @@ void KOViewManager::showTodoView()
     connectTodoView( mTodoView );
 
     KConfig *config = KOGlobals::self()->config();
-    mTodoView->restoreLayout( config, "Todo View" );
+    mTodoView->restoreLayout( config, "Todo View", false );
   }
   showView( mTodoView );
 }
@@ -527,7 +523,7 @@ void KOViewManager::setDocumentId( const QString &id )
 
 QWidget *KOViewManager::widgetForView( KOrg::BaseView *view ) const
 {
-  if ( ( view == mAgendaView || view == mAgendaSideBySideView ) && mAgendaViewTabs ) {
+  if ( mAgendaViewTabs && mAgendaViewTabs->indexOf( view ) >= 0 ) {
     return mAgendaViewTabs;
   }
   return view;
@@ -535,5 +531,16 @@ QWidget *KOViewManager::widgetForView( KOrg::BaseView *view ) const
 
 void KOViewManager::currentAgendaViewTabChanged( QWidget *widget )
 {
-  showView( static_cast<KOrg::BaseView*>( widget ) );
+  if ( widget )
+    showView( static_cast<KOrg::BaseView*>( widget ) );
+}
+
+void KOViewManager::setUpdateNeeded()
+{
+  if ( mAgendaView ) {
+    mAgendaView->setUpdateNeeded();
+  }
+  if ( mAgendaSideBySideView ) {
+    mAgendaSideBySideView->setUpdateNeeded();
+  }
 }
