@@ -49,6 +49,8 @@
 #include <QPixmap>
 #include <QBoxLayout>
 #include <QVBoxLayout>
+#include <QToolButton>
+#include <QHeaderView>
 
 using namespace KCal;
 
@@ -63,7 +65,7 @@ CalendarViewExtension *ResourceViewFactory::create( QWidget *parent )
   mResourceView = new ResourceView( mCalendar, parent );
 
   QObject::connect( mResourceView, SIGNAL(resourcesChanged()),
-                    mView, SLOT(updateView()) );
+                    mView, SLOT(resourcesChanged()) );
   QObject::connect( mResourceView, SIGNAL(resourcesChanged()),
                     mView, SLOT(updateCategories()) );
 
@@ -93,13 +95,13 @@ ResourceItem::ResourceItem( ResourceCalendar *resource, ResourceView *view,
   : QTreeWidgetItem( parent ),
     mResource( resource ), mView( view ), mBlockStateChange( false ),
     mIsSubresource( false ), mResourceIdentifier( QString() ),
-    mSubItemsCreated( false ), mIsStandardResource( false )
+    mSubItemsCreated( false ), mIsStandardResource( false ),
+    mIsReloading( false )
 {
   setFlags( flags() | Qt::ItemIsUserCheckable );
 
   setText( 0, resource->resourceName() );
-
-  mResourceColor = QColor();
+  mResourceColor = KOPrefs::instance()->resourceColor( resource->identifier() );
   setGuiState();
 
   if ( mResource->isActive() ) {
@@ -115,26 +117,48 @@ void ResourceItem::createSubresourceItems()
     // This resource has subresources
     QStringList::ConstIterator it;
     for ( it = subresources.begin(); it != subresources.end(); ++it ) {
-      ResourceItem *item = new ResourceItem( mResource, *it, mResource->labelForSubresource( *it ),
-                                             mView, this );
-      QColor resourceColor = KOPrefs::instance()->resourceColor( *it );
-      item->setResourceColor( resourceColor );
+      if ( !mView->findItemByIdentifier( *it ) ) {
+        new ResourceItem( mResource, *it, mResource->labelForSubresource( *it ),
+                          mView, this );
+      }
     }
   }
   mSubItemsCreated = true;
 }
 
+bool ResourceItem::useColors() const
+{
+  // assign a color, but only if this is a resource that actually
+  // hold items at top level
+  return KOPrefs::instance()->agendaViewColors() != KOPrefs::CategoryOnly  &&
+         ( mIsSubresource || ( !mIsReloading && mResource->subresources().isEmpty() ) ||
+           !mResource->canHaveSubresources() );
+}
+
+QVariant ResourceItem::data( int column, int role ) const
+{
+  if ( column == 0 &&
+       role == Qt::DecorationRole &&
+       mResourceColor.isValid() &&
+       useColors() ) {
+    return QVariant( mResourceColor );
+  } else {
+    return QTreeWidgetItem::data( column, role );
+  }
+}
+
 ResourceItem::ResourceItem( KCal::ResourceCalendar *resource,
                             const QString &sub, const QString &label,
                             ResourceView *view, ResourceItem *parent )
 
   : QTreeWidgetItem( parent ), mResource( resource ),
     mView( view ), mBlockStateChange( false ), mIsSubresource( true ),
-    mSubItemsCreated( false ), mIsStandardResource( false ), mActive( false )
+    mSubItemsCreated( false ), mIsStandardResource( false ), mActive( false ),
+    mIsReloading( false )
 {
   setFlags( flags() | Qt::ItemIsUserCheckable );
   setText( 0, label );
-  mResourceColor = QColor();
+  mResourceColor = KOPrefs::instance()->resourceColor( sub );
   mResourceIdentifier = sub;
   setGuiState();
 
@@ -183,10 +207,12 @@ void ResourceItem::stateChange( bool active )
         }
       }
     } else {
+      // mView->requestClose must be called before mResource->save() because 
+      // save causes closeResource do be called.
+      mView->requestClose( mResource );
       if ( mResource->save() ) {
         mResource->setActive( false );
       }
-      mView->requestClose( mResource );
     }
 
     setExpanded( mResource->isActive() && childCount() > 0 );
@@ -203,18 +229,7 @@ void ResourceItem::update()
 
 void ResourceItem::setResourceColor( QColor &color )
 {
-  if ( color.isValid() ) {
-    if ( mResourceColor != color ) {
-      int height = treeWidget()->visualItemRect(this).height();
-      QPixmap px( height - 4, height - 4 );
-      mResourceColor = color;
-      px.fill(color);
-      setData( 0, Qt::DecorationRole, px );
-    }
-  } else {
-    mResourceColor = color ;
-    setData( 0, Qt::DecorationRole, QPixmap() );
-  }
+  mResourceColor = color ;
 }
 
 void ResourceItem::setStandardResource( bool std )
@@ -230,6 +245,10 @@ ResourceView::ResourceView( KCal::CalendarResources *calendar, QWidget *parent )
   QBoxLayout *topLayout = new QVBoxLayout( this );
   topLayout->setSpacing( KDialog::spacingHint() );
 
+  QHBoxLayout *buttonBox = new QHBoxLayout();
+  buttonBox->setSpacing( KDialog::spacingHint() );
+  topLayout->addLayout( buttonBox );
+
   mListView = new QTreeWidget( this );
   mListView->setWhatsThis(
     i18n( "<qt><p>Select on this list the active KOrganizer "
@@ -246,17 +265,64 @@ ResourceView::ResourceView( KCal::CalendarResources *calendar, QWidget *parent )
           "to select the resource to use.</p></qt>" ) );
   mListView->setRootIsDecorated( false );
   mListView->setHeaderLabel( i18n( "Calendars" ) );
+  mListView->header()->hide();
   topLayout->addWidget( mListView );
 
+  mSelectedParent = 0;
+
   connect( mListView, SIGNAL(itemDoubleClicked(QTreeWidgetItem *,int)),
            SLOT(editResource()) );
+  connect( mListView, SIGNAL(itemClicked(QTreeWidgetItem *,int)),
+           SLOT(slotItemClicked(QTreeWidgetItem *,int)) );
+  connect( mListView, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)),
+           SLOT(currentChanged()) );
 
   mListView->setContextMenuPolicy( Qt::CustomContextMenu );
   connect( mListView, SIGNAL(customContextMenuRequested(const QPoint &)),
            SLOT(showContextMenu(const QPoint &)) );
 
-  connect( mListView, SIGNAL(itemClicked(QTreeWidgetItem *,int)),
-           SLOT(slotItemClicked(QTreeWidgetItem *,int)) );
+  QLabel *calLabel = new QLabel( i18n( "Calendar" ), this );
+  buttonBox->addWidget( calLabel );
+  buttonBox->addStretch( 1 );
+
+  mAddButton = new QToolButton( this );
+  mAddButton->setIcon( KIcon( "list-add" ) );
+  buttonBox->addWidget( mAddButton );
+  mAddButton->setToolTip( i18n( "Add calendar" ) );
+  mAddButton->setWhatsThis(
+                   i18n( "<qt><p>Press this button to add a resource to "
+                         "KOrganizer.</p>"
+                         "<p>Events, journal entries and to-dos are retrieved "
+                         "and stored on resources. Available "
+                         "resources include groupware servers, local files, "
+                         "journal entries as blogs on a server, etc... </p>"
+                         "<p>If you have more than one active resource, "
+                         "when creating incidents you will either automatically "
+                         "use the default resource or be prompted "
+                         "to select the resource to use.</p></qt>" ) );
+  mEditButton = new QToolButton( this );
+  mEditButton->setIcon( KIcon( "document-properties" ) );
+  buttonBox->addWidget( mEditButton );
+  mEditButton->setToolTip( i18n( "Edit calendar settings" ) );
+  mEditButton->setWhatsThis(
+                   i18n( "Press this button to edit the resource currently "
+                         "selected on the KOrganizer resources list above." ) );
+  mDeleteButton = new QToolButton( this );
+  mDeleteButton->setIcon( KIcon( "edit-delete" ) );
+  buttonBox->addWidget( mDeleteButton );
+  mDeleteButton->setToolTip( i18n( "Remove calendar" ) );
+  mDeleteButton->setWhatsThis(
+                   i18n( "Press this button to delete the resource currently "
+                         "selected on the KOrganizer resources list above." ) );
+  mDeleteButton->setDisabled( true );
+  mEditButton->setDisabled( true );
+
+  connect( mAddButton, SIGNAL( clicked() ), SLOT( slotAddButtonClicked() ) );
+  connect( mDeleteButton, SIGNAL( clicked() ), SLOT( removeResource() ) );
+  connect( mEditButton, SIGNAL( clicked() ), SLOT( editResource() ) );
+
+  setMinimumHeight( 50 );
+  mListView->setSortingEnabled( true );
 
   updateView();
 }
@@ -273,8 +339,12 @@ void ResourceView::updateView()
 
   KCal::CalendarResourceManager::Iterator it;
   for ( it = manager->begin(); it != manager->end(); ++it ) {
-    addResourceItem( *it );
+    addResourceItem( *it, false );
   }
+
+  mListView->sortItems( 0, Qt::AscendingOrder );
+
+  emit emitResourcesChanged();
 }
 
 void ResourceView::emitResourcesChanged()
@@ -283,13 +353,23 @@ void ResourceView::emitResourcesChanged()
   emit resourcesChanged();
 }
 
+void ResourceView::slotAddButtonClicked()
+{
+  if ( !mListView->selectedItems().isEmpty() ) {
+    mSelectedParent = static_cast<ResourceItem*>( mListView->selectedItems().first() );
+  } else {
+    mSelectedParent = 0;
+  }
+
+  addResource();
+}
+
 void ResourceView::addResource()
 {
   bool ok = false;
   KCal::CalendarResourceManager *manager = mCalendar->resourceManager();
-  ResourceItem *i = 0;
-  if ( !mListView->selectedItems().isEmpty() )
-    i = static_cast<ResourceItem*>( mListView->selectedItems().first() );
+  ResourceItem *i = mSelectedParent;
+
   if ( i && ( i->isSubresource() || i->resource()->canHaveSubresources() ) ) {
     const QString folderName =
       KInputDialog::getText( i18n( "Add Subresource" ),
@@ -339,32 +419,41 @@ void ResourceView::addResource()
 
   KOPrefs::instance()->setResourceColor( resource->identifier(), color );
 
-  if ( dlg && dlg->exec() ) {
+  bool success = true;
+  if ( !dlg || !dlg->exec() )
+    success = false;
+
+  if ( success ) {
     resource->setTimeSpec( KOPrefs::instance()->timeSpec() );
+    if ( resource->isActive() && ( !resource->open() || !resource->load() ) ) {
+      // ### There is a resourceLoadError() signal declared in ResourceCalendar
+      //     but no subclass seems to make use of it. We could do better.
+      KMessageBox::error( this, i18n("Unable to create the resource." ) );
+      success = false;
+    }
+  }
+
+  if ( success ) {
     manager->add( resource );
     // we have to call resourceAdded manually, because for in-process changes
     // the dcop signals are not connected, so the resource's signals would not
     // be connected otherwise
     mCalendar->resourceAdded( resource );
-  } else {
+  }
+
+  if ( !success ) {
     delete resource;
     resource = 0;
   }
   delete dlg;
+
+  //### maybe only do this if ( success )
   emitResourcesChanged();
 }
 
-void ResourceView::addResourceItem( ResourceCalendar *resource )
+void ResourceView::addResourceItem( ResourceCalendar *resource, bool emitSignal )
 {
-  ResourceItem *item = new ResourceItem( resource, this, mListView );
-
-  // assign a color, but only if this is a resource that actually
-  // hold items at top level
-  if ( !resource->canHaveSubresources() || resource->subresources().isEmpty() ) {
-    QColor resourceColor = KOPrefs::instance()->resourceColor( resource->identifier() );
-    item->setResourceColor( resourceColor );
-    item->update();
-  }
+  new ResourceItem( resource, this, mListView );
 
   connect( resource,
            SIGNAL(signalSubresourceAdded(ResourceCalendar *,const QString &,const QString &,const QString &)),
@@ -378,7 +467,9 @@ void ResourceView::addResourceItem( ResourceCalendar *resource )
            SLOT(closeResource(ResourceCalendar *)) );
 
   updateResourceList();
-  emit resourcesChanged();
+  if ( emitSignal ) {
+    emit resourcesChanged();
+  }
 }
 
 // Add a new entry
@@ -390,14 +481,12 @@ void ResourceView::slotSubresourceAdded( ResourceCalendar *calendar,
   Q_UNUSED( type );
   QList<QTreeWidgetItem *> items =
     mListView->findItems( calendar->resourceName(), Qt::MatchExactly, 0 );
-  if ( items.isEmpty() ) {
-    // Not found
-    return;
-  }
 
-  ResourceItem *item = static_cast<ResourceItem *>( items.first() );
-  ( void )new ResourceItem( calendar, resource, label, this, item );
-  emitResourcesChanged();
+  if ( !items.isEmpty() && !findItemByIdentifier( resource ) ) {
+    ResourceItem *item = static_cast<ResourceItem *>( items.first() );
+    ( void )new ResourceItem( calendar, resource, label, this, item );
+    emitResourcesChanged();
+  }
 }
 
 // Remove an entry
@@ -444,8 +533,8 @@ void ResourceView::removeResource()
   int km =
     KMessageBox::warningContinueCancel(
       this,
-      i18n( "<qt>Do you really want to delete the resource <b>%1</b>?</qt>",
-            item->text( 0 ) ), "", KStandardGuiItem::del() );
+      i18n( "<qt>Do you really want to remove the resource <b>%1</b>?</qt>",
+            item->text( 0 ) ), "", KStandardGuiItem::remove() );
   if ( km == KMessageBox::Cancel ) {
     return;
   }
@@ -453,17 +542,19 @@ void ResourceView::removeResource()
 // Don't be so restricitve
 #if 0
   if ( item->resource() == mCalendar->resourceManager()->standardResource() ) {
-    KMessageBox::sorry( this, i18n( "You cannot delete your standard resource." ) );
+    KMessageBox::sorry( this, i18n( "You cannot remove your standard resource." ) );
     return;
   }
 #endif
   if ( item->isSubresource() ) {
-    if ( !item->resource()->removeSubresource( item->resourceIdentifier() ) )
-      KMessageBox::sorry( this,
-              i18n ("<qt>Failed to remove the subresource <b>%1</b>. The "
-                  "reason could be that it is a built-in one which cannot "
-                  "be removed, or that the removal of the underlying storage "
-                  "folder failed.</qt>", item->text( 0 ) ) );
+    if ( !item->resource()->removeSubresource( item->resourceIdentifier() ) ) {
+      KMessageBox::sorry(
+        this,
+        i18n ( "<qt>Failed to remove the subresource <b>%1</b>. The "
+               "reason could be that it is a built-in one which cannot "
+               "be removed, or that the removal of the underlying storage "
+               "folder failed.</qt>", item->text( 0 ) ) );
+    }
     return;
   } else {
     mCalendar->resourceManager()->remove( item->resource() );
@@ -505,7 +596,8 @@ ResourceItem *ResourceView::findItem( ResourceCalendar *r )
 
 ResourceItem *ResourceView::findItemByIdentifier( const QString &id )
 {
-  QList<QTreeWidgetItem *> items = mListView->findItems( "*", Qt::MatchWildcard | Qt::MatchRecursive );
+  QList<QTreeWidgetItem *>items = mListView->findItems(
+    "*", Qt::MatchWildcard | Qt::MatchRecursive );
   foreach ( QTreeWidgetItem *i, items ) {
     ResourceItem *item = static_cast<ResourceItem *>( i );
     if ( item->resourceIdentifier() == id ) {
@@ -524,12 +616,14 @@ void ResourceView::showContextMenu( const QPoint &pos )
     QMenu *menu = new QMenu( this );
     menu->addAction( i18n( "&Add..." ), this, SLOT(addResource()) );
     menu->popup( mapToGlobal( pos ) );
+    mSelectedParent = 0;
 
     return;
   }
 
   KCal::CalendarResourceManager *manager = mCalendar->resourceManager();
   ResourceItem *item = static_cast<ResourceItem *>( i );
+  mSelectedParent = item;
 
   QMenu *menu = new QMenu( this );
   connect( menu, SIGNAL(aboutToHide()), menu, SLOT(deleteLater()) );
@@ -540,7 +634,7 @@ void ResourceView::showContextMenu( const QPoint &pos )
 
     QAction *saveAction = menu->addAction(
       i18nc( "save the resource", "&Save" ), this, SLOT(saveResource()) );
-    saveAction->setEnabled( item->resource()->isActive() );
+    saveAction->setEnabled( item->resource()->isActive() && !item->resource()->readOnly() );
 
     menu->addSeparator();
 
@@ -563,7 +657,7 @@ void ResourceView::showContextMenu( const QPoint &pos )
     menu->addSeparator();
   }
   menu->addAction( i18n( "&Add..." ), this, SLOT(addResource()) );
-  menu->popup( mapToGlobal( pos ) );
+  menu->popup( mListView->mapToGlobal( pos ) );
 }
 
 void ResourceView::assignColor()
@@ -628,9 +722,9 @@ void ResourceView::reloadResource()
   if ( !item ) {
     return;
   }
-
-  ResourceCalendar *r = item->resource();
-  r->load();
+  item->setIsReloading( true );
+  item->resource()->load();
+  item->setIsReloading( false );
 }
 
 void ResourceView::saveResource()
@@ -681,4 +775,12 @@ void ResourceView::slotItemClicked( QTreeWidgetItem *i, int )
   }
 }
 
+void ResourceView::currentChanged()
+{
+  ResourceItem *i = currentItem();
+
+  mDeleteButton->setEnabled( i != 0 );
+  mEditButton->setEnabled( i != 0 );
+}
+
 #include "resourceview.moc"
