@@ -35,6 +35,13 @@
 #include "koprefs.h"
 #include "koglobals.h"
 
+#include "kocorehelper.h"
+//#include "korganizer/baseview.h"
+//#include "actionmanager.h"
+#ifndef KORG_NOPRINTER
+#include "calprinter.h"
+#endif
+
 #include <kcal/todo.h>
 #include <kcal/incidence.h>
 
@@ -44,13 +51,14 @@
 #include <kiconloader.h>
 #include <klineedit.h>
 
-#include <QGridLayout>
 #include <QCheckBox>
-#include <QMenu>
-#include <QList>
 #include <QContextMenuEvent>
-#include <QModelIndex>
+#include <QGridLayout>
 #include <QHeaderView>
+#include <QList>
+#include <QMenu>
+#include <QTimer>
+#include <QModelIndex>
 
 using namespace KCal;
 using namespace KOrg;
@@ -114,8 +122,9 @@ KOTodoView::KOTodoView( Calendar *cal, QWidget *parent )
            SLOT(selectionChanged(const QItemSelection&, const QItemSelection&)) );
 
   mQuickAdd = new KOTodoViewQuickAddLine( this );
-  mQuickAdd->setClickMessage( i18n( "Click to add a new to-do" ) );
   mQuickAdd->setClearButtonShown( true );
+  QString clickText = i18n( "Click to add a new to-do" );
+  mQuickAdd->setToolTip( clickText );
   mQuickAdd->setVisible( KOPrefs::instance()->enableQuickTodo() );
   connect( mQuickAdd, SIGNAL(returnPressed(Qt::KeyboardModifiers)),
            this, SLOT(addQuickTodo(Qt::KeyboardModifiers)) );
@@ -125,7 +134,7 @@ KOTodoView::KOTodoView( Calendar *cal, QWidget *parent )
   mFlatView->setToolTip( i18n( "Display to-dos as list rather than as tree\n"
                                "(i.e. without parental relationship displayed)" ) );
   connect( mFlatView, SIGNAL(toggled(bool)),
-           mModel, SLOT(setFlatView(bool)) );
+           this, SLOT(setFlatView(bool)) );
 
   QGridLayout *layout = new QGridLayout( this );
   layout->addWidget( mQuickSearch, 0, 0, 1, 2 );
@@ -136,6 +145,12 @@ KOTodoView::KOTodoView( Calendar *cal, QWidget *parent )
 
   setLayout( layout );
 
+  // Do elided text after the layout is set so the width is properly computed
+  mQuickAdd->setClickMessage(
+    fontMetrics().elidedText(
+      clickText, Qt::ElideRight,
+      mQuickAdd->width() + mQuickAdd->clearButtonUsedSize().width() ) );
+
   // ---------------- POPUP-MENUS -----------------------
   mItemPopupMenu = new QMenu( this );
 
@@ -165,11 +180,15 @@ KOTodoView::KOTodoView( Calendar *cal, QWidget *parent )
   mItemPopupMenuItemOnlyEntries << mItemPopupMenu->addAction(
     i18n( "New Su&b-to-do..." ), this, SLOT(newSubTodo()) );
 
-  mItemPopupMenuItemOnlyEntries << mItemPopupMenu->addAction(
-    i18n( "&Make this To-do Independent" ), this, SIGNAL(unSubTodoSignal()) );
+  mMakeTodoIndependent = mItemPopupMenu->addAction( i18n( "&Make this To-do Independent" ),
+                                                    this, SIGNAL(unSubTodoSignal()) );
 
-  mItemPopupMenuItemOnlyEntries << mItemPopupMenu->addAction(
-    i18n( "Make all Sub-to-dos &Independent" ), this, SIGNAL(unAllSubTodoSignal()) );
+  mMakeSubtodosIndependent = mItemPopupMenu->addAction( i18n( "Make all Sub-to-dos &Independent" ),
+                                                        this, SIGNAL(unAllSubTodoSignal()) );
+
+  mItemPopupMenuItemOnlyEntries << mMakeTodoIndependent;
+
+  mItemPopupMenuItemOnlyEntries << mMakeSubtodosIndependent;
 
   mItemPopupMenu->addSeparator();
 
@@ -185,12 +204,49 @@ KOTodoView::KOTodoView( Calendar *cal, QWidget *parent )
   connect( mCopyPopupMenu, SIGNAL(dateChanged(QDate)),
            mItemPopupMenu, SLOT(hide()) );
 
+  mMovePopupMenu = new KDatePickerPopup( KDatePickerPopup::NoDate |
+                                         KDatePickerPopup::DatePicker |
+                                         KDatePickerPopup::Words,
+                                         QDate::currentDate(), this );
+  mMovePopupMenu->setTitle( i18n( "&Move To" ) );
+
+  connect( mMovePopupMenu, SIGNAL(dateChanged(const QDate &)),
+           SLOT(setNewDate(const QDate&)) );
+
+  connect( mMovePopupMenu, SIGNAL(dateChanged(QDate)),
+           mItemPopupMenu, SLOT(hide()) );
+
   mItemPopupMenu->insertMenu( 0, mCopyPopupMenu );
+  mItemPopupMenu->insertMenu( 0, mMovePopupMenu );
 
   mItemPopupMenu->addSeparator();
 
   mItemPopupMenu->addAction( i18nc( "delete completed to-dos", "Pur&ge Completed" ),
                              this, SIGNAL(purgeCompletedSignal()) );
+
+  mPriorityPopupMenu = new QMenu( this );
+  mPriority[ mPriorityPopupMenu->addAction( i18nc( "unspecified priority", "unspecified" ) ) ] = 0;
+  mPriority[ mPriorityPopupMenu->addAction( i18nc( "highest priority", "1 (highest)" ) ) ] = 1;
+  mPriority[ mPriorityPopupMenu->addAction( i18n( "2" ) ) ] = 2;
+  mPriority[ mPriorityPopupMenu->addAction( i18n( "3" ) ) ] = 3;
+  mPriority[ mPriorityPopupMenu->addAction( i18n( "4" ) ) ] = 4;
+  mPriority[ mPriorityPopupMenu->addAction( i18nc( "medium priority", "5 (medium)" ) ) ] = 5;
+  mPriority[ mPriorityPopupMenu->addAction( i18n( "6" ) ) ] = 6;
+  mPriority[ mPriorityPopupMenu->addAction( i18n( "7" ) ) ] = 7;
+  mPriority[ mPriorityPopupMenu->addAction( i18n( "8" ) ) ] = 8;
+  mPriority[ mPriorityPopupMenu->addAction( i18nc( "lowest priority", "9 (lowest)" ) ) ] = 9;
+  connect( mPriorityPopupMenu, SIGNAL(triggered(QAction *)),
+           SLOT(setNewPriority(QAction *)) );
+
+  mPercentageCompletedPopupMenu = new QMenu(this);
+  for ( int i = 0; i <= 100; i+=10 ) {
+    QString label = QString( "%1 %" ).arg( i );
+    mPercentage[mPercentageCompletedPopupMenu->addAction( label )] = i;
+  }
+  connect( mPercentageCompletedPopupMenu, SIGNAL(triggered(QAction *)),
+           SLOT(setNewPercentage(QAction *)) );
+
+  setMinimumHeight( 50 );
 }
 
 KOTodoView::~KOTodoView()
@@ -256,7 +312,7 @@ void KOTodoView::saveLayout( KConfig *config, const QString &group ) const
   cfgGroup.writeEntry( "FlatView", mFlatView->isChecked() );
 }
 
-void KOTodoView::restoreLayout( KConfig *config, const QString &group )
+void KOTodoView::restoreLayout( KConfig *config, const QString &group, bool minimalDefaults )
 {
   KConfigGroup cfgGroup = config->group( group );
   QHeaderView *header = mView->header();
@@ -264,20 +320,37 @@ void KOTodoView::restoreLayout( KConfig *config, const QString &group )
   QVariantList columnVisibility = cfgGroup.readEntry( "ColumnVisibility", QVariantList() );
   QVariantList columnOrder = cfgGroup.readEntry( "ColumnOrder", QVariantList() );
   QVariantList columnWidths = cfgGroup.readEntry( "ColumnWidths", QVariantList() );
-  for ( int i = 0;
-        i < header->count() && i < columnOrder.size() &&
-            i < columnWidths.size() && i < columnVisibility.size();
-        i++ ) {
-    bool visible = columnVisibility[i].toBool();
-    int width = columnWidths[i].toInt();
-    int order = columnOrder[i].toInt();
 
-    header->resizeSection( i, width );
-    header->moveSection( header->visualIndex( i ), order );
-    if ( i != 0 && !visible ) {
-      mView->hideColumn( i );
+  if ( columnVisibility.isEmpty() ) {
+    // if config is empty then use default settings
+    mView->hideColumn( eRecurColumn );
+
+    if ( minimalDefaults ) {
+      mView->hideColumn( ePriorityColumn );
+      mView->hideColumn( ePercentColumn );
+      mView->hideColumn( eDescriptionColumn );
+    }
+
+    // We don't have any incidences (content) yet, so we delay resizing
+    QTimer::singleShot( 0, this, SLOT(resizeColumnsToContent()) );
+
+  } else {
+      for ( int i = 0; i < header->count()     &&
+                       i < columnOrder.size()  &&
+                       i < columnWidths.size() &&
+                       i < columnVisibility.size(); i++ ) {
+      bool visible = columnVisibility[i].toBool();
+      int width = columnWidths[i].toInt();
+      int order = columnOrder[i].toInt();
+
+      header->resizeSection( i, width );
+      header->moveSection( header->visualIndex( i ), order );
+      if ( i != 0 && !visible ) {
+        mView->hideColumn( i );
+      }
     }
   }
+
   int sortOrder = cfgGroup.readEntry( "SortAscending", (int)Qt::AscendingOrder );
   int sortColumn = cfgGroup.readEntry( "SortColumn", -1 );
   if ( sortColumn >= 0 ) {
@@ -369,8 +442,35 @@ void KOTodoView::contextMenu( const QPoint &pos )
     entry->setEnabled( enable );
   }
   mCopyPopupMenu->setEnabled( enable );
+  mMovePopupMenu->setEnabled( enable );
 
-  mItemPopupMenu->popup( mView->viewport()->mapToGlobal( pos ) );
+  if ( enable ) {
+    Incidence::List incidences = selectedIncidences();
+    if ( !incidences.isEmpty() ) {
+      mMakeSubtodosIndependent->setEnabled( !incidences[0]->relations().isEmpty() );
+      mMakeTodoIndependent->setEnabled( incidences[0]->relatedTo() );
+    }
+
+    switch ( mView->indexAt( pos ).column() ) {
+    case ePriorityColumn:
+      mPriorityPopupMenu->popup( mView->viewport()->mapToGlobal( pos ) );
+      break;
+    case ePercentColumn:
+      mPercentageCompletedPopupMenu->popup( mView->viewport()->mapToGlobal( pos ) );
+      break;
+    case eDueDateColumn:
+      mMovePopupMenu->popup( mView->viewport()->mapToGlobal( pos ) );
+      break;
+    case eCategoriesColumn:
+      createCategoryPopupMenu()->popup( mView->viewport()->mapToGlobal( pos ) );
+      break;
+    default:
+      mItemPopupMenu->popup( mView->viewport()->mapToGlobal( pos ) );
+      break;
+    }
+  } else {
+    mItemPopupMenu->popup( mView->viewport()->mapToGlobal( pos ) );
+  }
 }
 
 void KOTodoView::selectionChanged( const QItemSelection &selected,
@@ -414,8 +514,32 @@ void KOTodoView::editTodo()
 
 void KOTodoView::printTodo()
 {
-  //TODO
-  kDebug() << "this is a stub";
+#ifndef KORG_NOPRINTER
+  QModelIndexList selection = mView->selectionModel()->selectedRows();
+  if ( selection.size() != 1 ) {
+    return;
+  }
+
+  Todo *todo = static_cast<Todo *>( selection[0].data( KOTodoModel::TodoRole ).value<void *>() );
+
+  KOCoreHelper helper;
+  CalPrinter printer( this, BaseView::calendar(), &helper );
+  connect( this, SIGNAL(configChanged()), &printer, SLOT(updateConfig()) );
+
+  Incidence::List selectedIncidences;
+  selectedIncidences.append( todo );
+
+  KDateTime todoDate;
+  if ( todo->hasStartDate() ) {
+    todoDate = todo->dtStart();
+  } else {
+    todoDate = todo->dtDue();
+  }
+
+  printer.print( KOrg::CalPrinterBase::Incidence,
+                 todoDate.date(), todoDate.date(), selectedIncidences, false );
+
+#endif
 }
 
 void KOTodoView::deleteTodo()
@@ -469,4 +593,160 @@ void KOTodoView::itemDoubleClicked( const QModelIndex &index )
   }
 }
 
+QMenu *KOTodoView::createCategoryPopupMenu()
+{
+  QMenu *tempMenu = new QMenu( this );
+
+  QModelIndexList selection = mView->selectionModel()->selectedRows();
+  if ( selection.size() != 1 ) {
+    return tempMenu;
+  }
+
+  Todo *todo = static_cast<Todo *>( selection[0].data( KOTodoModel::TodoRole ).value<void *>() );
+  QStringList checkedCategories = todo->categories();
+
+  QStringList::Iterator it;
+  for ( it = KOPrefs::instance()->mCustomCategories.begin();
+        it != KOPrefs::instance()->mCustomCategories.end();
+        ++it ) {
+    QAction *action = tempMenu->addAction( *it );
+    action->setCheckable(true);
+    mCategory[ action ] = *it;
+    if ( checkedCategories.contains( *it ) ) {
+      action->setChecked( true );
+    }
+  }
+
+  connect( tempMenu, SIGNAL(triggered(QAction *)),
+           SLOT(changedCategories(QAction *)) );
+  connect( tempMenu, SIGNAL(aboutToHide()),
+           tempMenu, SLOT(deleteLater()) );
+  return tempMenu;
+}
+
+void KOTodoView::setNewDate( const QDate &date )
+{
+  QModelIndexList selection = mView->selectionModel()->selectedRows();
+  if ( selection.size() != 1 ) {
+    return;
+  }
+
+  Todo *todo = static_cast<Todo *>( selection[0].data( KOTodoModel::TodoRole ).value<void *>() );
+  if ( !todo->isReadOnly() && mChanger->beginChange( todo ) ) {
+    Todo *oldTodo = todo->clone();
+
+    KDateTime dt( date );
+
+    if ( !todo->allDay() ) {
+      dt.setTime( todo->dtDue().time() );
+    }
+
+    if ( date.isNull() ) {
+      todo->setHasDueDate( false );
+    } else if ( !todo->hasDueDate() ) {
+      todo->setHasDueDate( true );
+    }
+    todo->setDtDue( dt );
+
+    mChanger->changeIncidence( oldTodo, todo, KOGlobals::COMPLETION_MODIFIED );
+    mChanger->endChange( todo );
+    delete oldTodo;
+  } else {
+    kDebug() << "No active item, active item is read-only, or locking failed";
+  }
+}
+
+void KOTodoView::setNewPercentage( QAction *action )
+{
+  QModelIndexList selection = mView->selectionModel()->selectedRows();
+  if ( selection.size() != 1 ) {
+    return;
+  }
+
+  Todo *todo = static_cast<Todo *>( selection[0].data( KOTodoModel::TodoRole ).value<void *>() );
+  if ( !todo->isReadOnly() && mChanger->beginChange( todo ) ) {
+    Todo *oldTodo = todo->clone();
+
+    int percentage = mPercentage[action];
+    if ( percentage == 100 ) {
+      todo->setCompleted( KDateTime::currentLocalDateTime() );
+      todo->setPercentComplete( percentage );
+    } else {
+      todo->setCompleted( false );
+      todo->setPercentComplete( percentage );
+    }
+    if ( todo->recurs() && percentage == 100 ) {
+      mChanger->changeIncidence( oldTodo, todo, KOGlobals::COMPLETION_MODIFIED_WITH_RECURRENCE );
+    } else {
+      mChanger->changeIncidence( oldTodo, todo, KOGlobals::COMPLETION_MODIFIED );
+    }
+    mChanger->endChange( todo );
+    delete oldTodo;
+  } else {
+    kDebug() << "No active item, active item is read-only, or locking failed";
+  }
+}
+
+void KOTodoView::setNewPriority( QAction *action )
+{
+  QModelIndexList selection = mView->selectionModel()->selectedRows();
+  if ( selection.size() != 1 ) {
+    return;
+  }
+
+  Todo *todo = static_cast<Todo *>( selection[0].data( KOTodoModel::TodoRole ).value<void *>() );
+  if ( !todo->isReadOnly() &&
+       mChanger->beginChange( todo ) ) {
+    Todo *oldTodo = todo->clone();
+    todo->setPriority( mPriority[action] );
+
+    mChanger->changeIncidence( oldTodo, todo, KOGlobals::PRIORITY_MODIFIED );
+    mChanger->endChange( todo );
+    delete oldTodo;
+  }
+}
+
+void KOTodoView::changedCategories( QAction *action )
+{
+  QModelIndexList selection = mView->selectionModel()->selectedRows();
+  if ( selection.size() != 1 ) {
+    return;
+  }
+
+  Todo *todo = static_cast<Todo *>( selection[0].data( KOTodoModel::TodoRole ).value<void *>() );
+  if ( !todo->isReadOnly() && mChanger->beginChange( todo ) ) {
+    Todo *oldTodo = todo->clone();
+
+    QStringList categories = todo->categories();
+    if ( categories.contains( mCategory[action] ) ) {
+      categories.removeAll( mCategory[action] );
+    } else {
+      categories.append( mCategory[action] );
+    }
+    categories.sort();
+    todo->setCategories( categories );
+    mChanger->changeIncidence( oldTodo, todo, KOGlobals::CATEGORY_MODIFIED );
+    mChanger->endChange( todo );
+    delete oldTodo;
+  } else {
+    kDebug() << "No active item, active item is read-only, or locking failed";
+  }
+}
+
+void KOTodoView::setFlatView( bool flatView ) {
+  mView->setRootIsDecorated( !flatView );
+  mModel->setFlatView( flatView );
+}
+
+bool KOTodoView::usesFullWindow()
+{
+  return KOPrefs::instance()->mFullViewTodo;
+}
+
+void KOTodoView::resizeColumnsToContent()
+{
+  mView->resizeColumnToContents( eDueDateColumn );
+  mView->resizeColumnToContents( eSummaryColumn );
+}
+
 #include "kotodoview.moc"
