@@ -1,6 +1,6 @@
 /* KPilot
 **
-** Copyright (C) 2002-2003 by Reinhold Kainhofer
+** Copyright (C) 2002-2003 by Reinhold Kainhofer <reinhold@kainhofer.com>
 **
 */
 
@@ -17,46 +17,38 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-** MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
 */
 
 /*
 ** Bug reports and questions can be sent to kde-pim@kde.org.
 */
 
+#include "time-conduit.h"
+
 #include "options.h"
 
 #include <time.h>
 
+#include <pilotSysInfo.h>
+
 #include <kconfig.h>
 #include <kdebug.h>
 
-#include "time-factory.h"
-#include "time-conduit.h"
 #include "timeConduitSettings.h"
 
-
-// Something to allow us to check what revision
-// the modules are that make up a binary distribution.
-extern "C"
-{
-long version_conduit_time = KPILOT_PLUGIN_API ;
-const char *id_conduit_time =
-	"$Id: time-conduit.cc,v 1.19 2005/02/06 14:47:43 adridg Exp $";
-}
+#define DIR_PCToPalm 0
+#define DIR_PalmToPC 1
 
 
 
-TimeConduit::TimeConduit(KPilotDeviceLink * o,
-	const char *n,
-	const QStringList & a) :
-	ConduitAction(o, n, a)
+
+TimeConduit::TimeConduit(KPilotLink * o,
+	const QVariantList & a) :
+	ConduitAction(o, "Time", a)
 {
 	FUNCTIONSETUP;
-#ifdef DEBUG
-	DEBUGCONDUIT<<id_conduit_time<<endl;
-#endif
 	fConduitName=i18n("Time");
 }
 
@@ -79,10 +71,14 @@ void TimeConduit::readConfig()
 /* virtual */ bool TimeConduit::exec()
 {
 	FUNCTIONSETUP;
-	DEBUGCONDUIT<<id_conduit_time<<endl;
 
 	readConfig();
 
+	if (syncMode().isLocal())
+	{
+		return delayDone();
+	}
+
 	emit logMessage(i18n("Setting the clock on the handheld"));
 	syncHHfromPC();
 	return delayDone();
@@ -94,9 +90,9 @@ void TimeConduit::syncHHfromPC()
 	FUNCTIONSETUP;
 	time_t ltime;
 	time(&ltime);
-	QDateTime time=QDateTime::currentDateTime();
 
-	long int major=fHandle->majorVersion(), minor=fHandle->minorVersion();
+	long int major=fHandle->getSysInfo().getMajorVersion(),
+		 minor=fHandle->getSysInfo().getMinorVersion();
 
 	if (major==3 && (minor==25 || minor==30))
 	{
@@ -104,9 +100,13 @@ void TimeConduit::syncHHfromPC()
 		return;
 	}
 
-	fHandle->setTime(ltime);
-#ifdef DEBUG
-	time.setTime_t(ltime);
-	DEBUGCONDUIT<<fname<<": synced time "<<time.toString()<<" to the handheld"<<endl;
-#endif
+	int sd = pilotSocket();
+	if ( sd > 0 )
+	{
+		dlp_SetSysDateTime( sd, ltime );
+	}
+	else
+	{
+		WARNINGKPILOT <<"Link is not a real device.";
+	}
 }
