@@ -1,6 +1,6 @@
 /* KPilot
 **
-** Copyright (C) 1998-2001 by Dan Pilone
+** Copyright (C) 1998-2001 by Dan Pilone <dan@kpilot.org>
 ** Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 ** Copyright (C) 2005-2006 Adriaan de Groot <groot@kde.org>
 **
@@ -29,16 +29,16 @@
 ** Bug reports and questions can be sent to kde-pim@kde.org
 */
 
-#include "options.h"
+#include "pilotDatabase.h"
 
 #include <time.h> // Needed by pilot-link include
 #include <pi-appinfo.h>
 
-#include <qstringlist.h>
+#include <QtCore/QStringList>
 
 #include <kglobal.h>
 
-#include "pilotDatabase.h"
+#include "options.h"
 #include "pilotRecord.h"
 
 static int creationCount = 0;
@@ -63,18 +63,17 @@ PilotDatabase::PilotDatabase(const QString &s) :
 	creationCount--;
 	if (createdNames)
 	{
-		createdNames->remove(fName.isEmpty() ? CSL1("<empty>") : fName);
+		createdNames->removeAll(fName.isEmpty() ? CSL1("<empty>") : fName);
 	}
 }
 
 /* static */ int PilotDatabase::instanceCount()
 {
 	FUNCTIONSETUP;
-	DEBUGKPILOT << fname << ": " << creationCount << " databases." << endl;
+	DEBUGKPILOT << creationCount << " databases.";
 	if (createdNames)
 	{
-		DEBUGKPILOT << fname << ": "
-			<< createdNames->join(CSL1(",")) << endl;
+		DEBUGKPILOT << createdNames->join(CSL1(","));
 	}
 	return creationCount;
 }
@@ -83,10 +82,9 @@ PilotDatabase::PilotDatabase(const QString &s) :
 {
 	Pilot::RecordIDList l;
 
-	for (unsigned int i = 0 ; ; i++)
-	{
-		PilotRecord *r = readRecordByIndex(i);
-		if (!r) break;
+  unsigned int i = 0;
+	while (PilotRecord* r = readRecordByIndex( i++ ))
+  {
 		l.append(r->id());
 		delete r;
 	}
