@@ -34,7 +34,7 @@
 #include "resourcekolab.h"
 
 #include <kabc/addressee.h>
-#include <kabc/stdaddressbook.h>
+#include <kabc/addressbook.h>
 #include <libkdepim/distributionlist.h>
 #include <kio/netaccess.h>
 #include <kdebug.h>
@@ -49,10 +49,10 @@ static const char* s_soundAttachmentName = "sound";
 static const char* s_unhandledTagAppName = "KOLABUNHANDLED"; // no hyphens in appnames!
 
 // saving (addressee->xml)
-Contact::Contact( const KABC::Addressee* addr )
+Contact::Contact( const KABC::Addressee* addr, KABC::AddressBook* addressBook )
   : mHasGeo( false )
 {
-  setFields( addr );
+  setFields( addr, addressBook );
 }
 
 // loading (xml->addressee)
@@ -222,15 +222,15 @@ QString Contact::profession() const
   return mProfession;
 }
 
-// void Contact::setJobTitle( const QString& title )
-// {
-//   mJobTitle = title;
-// }
+void Contact::setTitle( const QString& title )
+{
+  mTitle = title;
+}
 
-// QString Contact::jobTitle() const
-// {
-//   return mJobTitle;
-// }
+QString Contact::title() const
+{
+  return mTitle;
+}
 
 void Contact::setManagerName( const QString& name )
 {
@@ -451,8 +451,8 @@ bool Contact::loadPhoneAttribute( QDomElement& element )
 
 void Contact::savePhoneAttributes( QDomElement& element ) const
 {
-  QList<PhoneNumber>::ConstIterator it = mPhoneNumbers.begin();
-  for ( ; it != mPhoneNumbers.end(); ++it ) {
+  QList<PhoneNumber>::ConstIterator it = mPhoneNumbers.constBegin();
+  for ( ; it != mPhoneNumbers.constEnd(); ++it ) {
     QDomElement e = element.ownerDocument().createElement( "phone" );
     element.appendChild( e );
     const PhoneNumber& p = *it;
@@ -463,8 +463,8 @@ void Contact::savePhoneAttributes( QDomElement& element ) const
 
 void Contact::saveEmailAttributes( QDomElement& element ) const
 {
-  QList<Email>::ConstIterator it = mEmails.begin();
-  for ( ; it != mEmails.end(); ++it )
+  QList<Email>::ConstIterator it = mEmails.constBegin();
+  for ( ; it != mEmails.constEnd(); ++it )
     saveEmailAttribute( element, *it );
 }
 
@@ -479,8 +479,8 @@ void Contact::loadCustomAttributes( QDomElement& element )
 
 void Contact::saveCustomAttributes( QDomElement& element ) const
 {
-  QList<Custom>::ConstIterator it = mCustomList.begin();
-  for ( ; it != mCustomList.end(); ++it ) {
+  QList<Custom>::ConstIterator it = mCustomList.constBegin();
+  for ( ; it != mCustomList.constEnd(); ++it ) {
     Q_ASSERT( !(*it).name.isEmpty() );
     if ( (*it).app == s_unhandledTagAppName ) {
       writeString( element, (*it).name, (*it).value );
@@ -535,8 +535,8 @@ bool Contact::loadAddressAttribute( QDomElement& element )
 
 void Contact::saveAddressAttributes( QDomElement& element ) const
 {
-  QList<Address>::ConstIterator it = mAddresses.begin();
-  for ( ; it != mAddresses.end(); ++it ) {
+  QList<Address>::ConstIterator it = mAddresses.constBegin();
+  for ( ; it != mAddresses.constEnd(); ++it ) {
     QDomElement e = element.ownerDocument().createElement( "address" );
     element.appendChild( e );
     const Address& a = *it;
@@ -578,8 +578,8 @@ void Kolab::Contact::loadDistrListMember( const QDomElement& element )
 
 void Contact::saveDistrListMembers( QDomElement& element ) const
 {
-  QList<Member>::ConstIterator it = mDistrListMembers.begin();
-  for( ; it != mDistrListMembers.end(); ++it ) {
+  QList<Member>::ConstIterator it = mDistrListMembers.constBegin();
+  for( ; it != mDistrListMembers.constEnd(); ++it ) {
     QDomElement e = element.ownerDocument().createElement( "member" );
     element.appendChild( e );
     const Member& m = *it;
@@ -657,9 +657,9 @@ bool Contact::loadAttribute( QDomElement& element )
     }
     break;
   case 'j':
-    if ( tagName == "job-title" ) {
-      // see saveAttributes: <job-title> is mapped to the Role field
-      setRole( element.text() );
+   if ( tagName == "job-title" ) {
+     // see saveAttributes: <job-title> is mapped to the Role field
+      setTitle( element.text() );
       return true;
     }
     break;
@@ -749,6 +749,10 @@ bool Contact::loadAttribute( QDomElement& element )
       loadCustomAttributes( element );
       return true;
     }
+    if ( tagName == "x-title" ) {
+      setTitle( element.text() );
+      return true;
+    }
     break;
   case 'w':
     if ( tagName == "web-page" ) {
@@ -766,7 +770,6 @@ bool Contact::saveAttributes( QDomElement& element ) const
 {
   // Save the base class elements
   KolabBase::saveAttributes( element );
-
   if ( mIsDistributionList ) {
     writeString( element, "display-name", fullName() );
     saveDistrListMembers( element );
@@ -779,11 +782,8 @@ bool Contact::saveAttributes( QDomElement& element ) const
     writeString( element, "department", department() );
     writeString( element, "office-location", officeLocation() );
     writeString( element, "profession", profession() );
-    // <role> is gone; jobTitle() is not shown in the addresseeeditor,
-    // so let's bind <job-title> to role()
-    //writeString( element, "role", role() );
-    //writeString( element, "job-title", jobTitle() );
-    writeString( element, "job-title", role() );
+    writeString( element, "role", role() );
+    writeString( element, "job-title", title() );
     writeString( element, "manager-name", managerName() );
     writeString( element, "assistant", assistant() );
     writeString( element, "nick-name", nickName() );
@@ -891,28 +891,28 @@ static QStringList phoneTypeToString( KABC::PhoneNumber::Type type )
     type = type & ~KABC::PhoneNumber::Work;
   }
 
-  // To support both "home1" and "home2", map Home+Pref to home1
+  // To support both "home1" and "home2", map Home+Pref to home2
   if ( ( type & KABC::PhoneNumber::Home ) && ( type & KABC::PhoneNumber::Pref ) )
   {
-      types << "home1";
+      types << "home2";
       type = type & ~KABC::PhoneNumber::Home;
       type = type & ~KABC::PhoneNumber::Pref;
   }
-  // To support both "business1" and "business2", map Work+Pref to business1
+  // To support both "business1" and "business2", map Work+Pref to business2
   if ( ( type & KABC::PhoneNumber::Work ) && ( type & KABC::PhoneNumber::Pref ) )
   {
-      types << "business1";
+      types << "business2";
       type = type & ~KABC::PhoneNumber::Work;
       type = type & ~KABC::PhoneNumber::Pref;
   }
 
 
   if ( type & KABC::PhoneNumber::Home )
-    types << "home2";
+    types << "home1";
   if ( type & KABC::PhoneNumber::Msg ) // Msg==messaging
     types << "company";
   if ( type & KABC::PhoneNumber::Work )
-    types << "business2";
+    types << "business1";
   if ( type & KABC::PhoneNumber::Pref )
     types << "primary";
   if ( type & KABC::PhoneNumber::Voice )
@@ -942,13 +942,13 @@ static KABC::PhoneNumber::Type phoneTypeFromString( const QString& type )
     return KABC::PhoneNumber::Home | KABC::PhoneNumber::Fax;
   if ( type == "businessfax" )
     return KABC::PhoneNumber::Work | KABC::PhoneNumber::Fax;
-  if ( type == "business1" )
-    return KABC::PhoneNumber::Work | KABC::PhoneNumber::Pref;
   if ( type == "business2" )
+    return KABC::PhoneNumber::Work | KABC::PhoneNumber::Pref;
+  if ( type == "business1" )
     return KABC::PhoneNumber::Work;
-  if ( type == "home1" )
-    return KABC::PhoneNumber::Home | KABC::PhoneNumber::Pref;
   if ( type == "home2" )
+    return KABC::PhoneNumber::Home | KABC::PhoneNumber::Pref;
+  if ( type == "home1" )
     return KABC::PhoneNumber::Home;
   if ( type == "company" )
     return KABC::PhoneNumber::Msg;
@@ -988,7 +988,7 @@ static const char* s_knownCustomFields[] = {
 };
 
 // The saving is addressee -> Contact -> xml, this is the first part
-void Contact::setFields( const KABC::Addressee* addressee )
+void Contact::setFields( const KABC::Addressee* addressee, KABC::AddressBook* addressBook )
 {
   KolabBase::setFields( addressee );
 
@@ -997,9 +997,9 @@ void Contact::setFields( const KABC::Addressee* addressee )
     // Hopefully all resources are available during saving, so we can look up
     // in the addressbook to get name+email from the UID.
     KPIM::DistributionList distrList( *addressee );
-    const KPIM::DistributionList::Entry::List entries = distrList.entries( KABC::StdAddressBook::self() );
-    KPIM::DistributionList::Entry::List::ConstIterator it = entries.begin();
-    for ( ; it != entries.end() ; ++it ) {
+    const KPIM::DistributionList::Entry::List entries = distrList.entries( addressBook );
+    KPIM::DistributionList::Entry::List::ConstIterator it = entries.constBegin();
+    for ( ; it != entries.constEnd() ; ++it ) {
       Member m;
       m.displayName = (*it).addressee.formattedName();
       m.email = (*it).email;
@@ -1018,11 +1018,15 @@ void Contact::setFields( const KABC::Addressee* addressee )
   setOrganization( addressee->organization() );
   setWebPage( addressee->url().url() );
   setIMAddress( addressee->custom( "KADDRESSBOOK", "X-IMAddress" ) );
-  setDepartment( addressee->department() );
+#if KDE_IS_VERSION(3,5,8)
+  setDepartment( addressee->department());
+#else
+  setDepartment( addressee->custom( "KADDRESSBOOK", "X-Department" ) );
+#endif
   setOfficeLocation( addressee->custom( "KADDRESSBOOK", "X-Office" ) );
   setProfession( addressee->custom( "KADDRESSBOOK", "X-Profession" ) );
   setRole( addressee->role() );
-  //setJobTitle( addressee->title() );
+  setTitle( addressee->title() );
   setManagerName( addressee->custom( "KADDRESSBOOK", "X-ManagersName" ) );
   setAssistant( addressee->custom( "KADDRESSBOOK", "X-AssistantsName" ) );
   setNickName( addressee->nickName() );
@@ -1037,7 +1041,7 @@ void Contact::setFields( const KABC::Addressee* addressee )
   // Conversion problem here:
   // KABC::Addressee has only one full name and N addresses, but the XML format
   // has N times (fullname+address). So we just copy the fullname over and ignore it on loading.
-  for ( QStringList::ConstIterator it = emails.begin(); it != emails.end(); ++it ) {
+  for ( QStringList::ConstIterator it = emails.constBegin(); it != emails.constEnd(); ++it ) {
     Email email;
     email.displayName = fullName();
     email.smtpAddress = *it;
@@ -1047,7 +1051,7 @@ void Contact::setFields( const KABC::Addressee* addressee )
   // Now the real-world addresses
   QString preferredAddress = "home";
   const KABC::Address::List addresses = addressee->addresses();
-  for ( KABC::Address::List::ConstIterator it = addresses.begin() ; it != addresses.end(); ++it ) {
+  for ( KABC::Address::List::ConstIterator it = addresses.constBegin() ; it != addresses.constEnd(); ++it ) {
     Address address;
     address.kdeAddressType = (*it).type();
     address.type = addressTypeToString( address.kdeAddressType );
@@ -1067,10 +1071,10 @@ void Contact::setFields( const KABC::Addressee* addressee )
   setPreferredAddress( preferredAddress );
 
   const KABC::PhoneNumber::List phones = addressee->phoneNumbers();
-  for ( KABC::PhoneNumber::List::ConstIterator it = phones.begin(); it != phones.end(); ++it ) {
+  for ( KABC::PhoneNumber::List::ConstIterator it = phones.constBegin(); it != phones.constEnd(); ++it ) {
     // Create a tag per phone type set in the bitfield
     QStringList types = phoneTypeToString( (*it).type() );
-    for( QStringList::Iterator typit = types.begin(); typit != types.end(); ++typit ) {
+    for( QStringList::ConstIterator typit = types.constBegin(); typit != types.constEnd(); ++typit ) {
       PhoneNumber phoneNumber;
       phoneNumber.type = *typit;
       phoneNumber.number = (*it).number();
@@ -1105,7 +1109,7 @@ void Contact::setFields( const KABC::Addressee* addressee )
   for ( const char** p = s_knownCustomFields; *p; ++p )
     knownCustoms << QString::fromLatin1( *p );
   QStringList customs = addressee->customs();
-  for( QStringList::Iterator it = customs.begin(); it != customs.end(); ++it ) {
+  for( QStringList::ConstIterator it = customs.constBegin(); it != customs.constEnd(); ++it ) {
     // KABC::Addressee doesn't offer a real way to iterate over customs, other than splitting strings ourselves
     // The format is "app-name:value".
     int pos = (*it).indexOf( '-' );
@@ -1149,8 +1153,8 @@ void Contact::saveTo( KABC::Addressee* addressee )
   if ( mIsDistributionList ) {
     KPIM::DistributionList distrList( *addressee );
     distrList.setName( fullName() );
-    QList<Member>::ConstIterator mit = mDistrListMembers.begin();
-    for ( ; mit != mDistrListMembers.end(); ++mit ) {
+    QList<Member>::ConstIterator mit = mDistrListMembers.constBegin();
+    for ( ; mit != mDistrListMembers.constEnd(); ++mit ) {
       QString displayName = (*mit).displayName;
       // fixup the display name DistributionList::assumes neither ',' nor ';' is present
       displayName.replace( ',', ' ' );
@@ -1172,11 +1176,15 @@ void Contact::saveTo( KABC::Addressee* addressee )
   addressee->setOrganization( organization() );
   addressee->setUrl( webPage() );
   addressee->insertCustom( "KADDRESSBOOK", "X-IMAddress", imAddress() );
+#if KDE_IS_VERSION(3,5,8)
   addressee->setDepartment( department() );
+#else
+  addressee->insertCustom( "KADDRESSBOOK", "X-Department", department() );
+#endif
   addressee->insertCustom( "KADDRESSBOOK", "X-Office", officeLocation() );
   addressee->insertCustom( "KADDRESSBOOK", "X-Profession", profession() );
   addressee->setRole( role() );
-  //addressee->setTitle( jobTitle() );
+  addressee->setTitle( title() );
   addressee->insertCustom( "KADDRESSBOOK", "X-ManagersName", managerName() );
   addressee->insertCustom( "KADDRESSBOOK", "X-AssistantsName", assistant() );
   addressee->setNickName( nickName() );
@@ -1208,13 +1216,13 @@ void Contact::saveTo( KABC::Addressee* addressee )
     addressee->setGeo( KABC::Geo( mLatitude, mLongitude ) );
 
   QStringList emailAddresses;
-  for ( QList<Email>::ConstIterator it = mEmails.begin(); it != mEmails.end(); ++it ) {
+  for ( QList<Email>::ConstIterator it = mEmails.constBegin(); it != mEmails.constEnd(); ++it ) {
     // we can't do anything with (*it).displayName
     emailAddresses.append( (*it).smtpAddress );
   }
   addressee->setEmails( emailAddresses );
 
-  for ( QList<Address>::ConstIterator it = mAddresses.begin(); it != mAddresses.end(); ++it ) {
+  for ( QList<Address>::ConstIterator it = mAddresses.constBegin(); it != mAddresses.constEnd(); ++it ) {
     KABC::Address address;
     int type = (*it).kdeAddressType;
     if ( type == -1 ) { // no kde-specific type available
@@ -1232,14 +1240,14 @@ void Contact::saveTo( KABC::Addressee* addressee )
     addressee->insertAddress( address );
   }
 
-  for ( QList<PhoneNumber>::ConstIterator it = mPhoneNumbers.begin(); it != mPhoneNumbers.end(); ++it ) {
+  for ( QList<PhoneNumber>::ConstIterator it = mPhoneNumbers.constBegin(); it != mPhoneNumbers.constEnd(); ++it ) {
     KABC::PhoneNumber number;
     number.setType( phoneTypeFromString( (*it).type ) );
     number.setNumber( (*it).number );
     addressee->insertPhoneNumber( number );
   }
 
-  for( QList<Custom>::ConstIterator it = mCustomList.begin(); it != mCustomList.end(); ++it ) {
+  for( QList<Custom>::ConstIterator it = mCustomList.constBegin(); it != mCustomList.constEnd(); ++it ) {
     QString app = (*it).app.isEmpty() ? QString::fromLatin1( "KADDRESSBOOK" ) : (*it).app;
     addressee->insertCustom( app, (*it).name, (*it).value );
   }
